from pymammotion.proto import common_pb2 as _common_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AppGetAllAreaHashName(_message.Message):
    __slots__ = ["deviceId", "hashnames"]
    DEVICEID_FIELD_NUMBER: _ClassVar[int]
    HASHNAMES_FIELD_NUMBER: _ClassVar[int]
    deviceId: str
    hashnames: _containers.RepeatedCompositeFieldContainer[area_hash_name]
    def __init__(self, deviceId: _Optional[str] = ..., hashnames: _Optional[_Iterable[_Union[area_hash_name, _Mapping]]] = ...) -> None: ...

class ManualElementMessage(_message.Message):
    __slots__ = ["dataHash", "ifHide", "point1_center_x", "point1_center_y", "point2_height_y", "point2_width_x", "pver", "reserved", "result", "rotate_radius", "shape", "subCmd", "type"]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    IFHIDE_FIELD_NUMBER: _ClassVar[int]
    POINT1_CENTER_X_FIELD_NUMBER: _ClassVar[int]
    POINT1_CENTER_Y_FIELD_NUMBER: _ClassVar[int]
    POINT2_HEIGHT_Y_FIELD_NUMBER: _ClassVar[int]
    POINT2_WIDTH_X_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    ROTATE_RADIUS_FIELD_NUMBER: _ClassVar[int]
    SHAPE_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    dataHash: int
    ifHide: bool
    point1_center_x: float
    point1_center_y: float
    point2_height_y: float
    point2_width_x: float
    pver: int
    reserved: str
    result: int
    rotate_radius: float
    shape: int
    subCmd: int
    type: int
    def __init__(self, pver: _Optional[int] = ..., type: _Optional[int] = ..., shape: _Optional[int] = ..., result: _Optional[int] = ..., subCmd: _Optional[int] = ..., dataHash: _Optional[int] = ..., ifHide: bool = ..., reserved: _Optional[str] = ..., point1_center_x: _Optional[float] = ..., point1_center_y: _Optional[float] = ..., point2_width_x: _Optional[float] = ..., point2_height_y: _Optional[float] = ..., rotate_radius: _Optional[float] = ...) -> None: ...

class MctlNav(_message.Message):
    __slots__ = ["all_plan_task", "app_request_cover_paths", "bidire_reqconver_path", "bidire_taskid", "cover_path_upload", "nav_sys_param_cmd", "plan_task_execute", "plan_task_name_id", "simulation_cmd", "toapp_all_hash_name", "toapp_bp", "toapp_bstate", "toapp_chgpileto", "toapp_costmap", "toapp_edge_points", "toapp_edge_points_ack", "toapp_get_commondata_ack", "toapp_gethash_ack", "toapp_lat_up", "toapp_manual_element", "toapp_map_name_msg", "toapp_opt_border_info", "toapp_opt_line_up", "toapp_opt_obs_info", "toapp_pos_up", "toapp_svg_msg", "toapp_task_info", "toapp_work_report_ack", "toapp_work_report_update_ack", "toapp_work_report_upload", "toapp_zigzag", "todev_cancel_draw_cmd", "todev_cancel_suscmd", "todev_chl_line", "todev_chl_line_data", "todev_chl_line_end", "todev_draw_border", "todev_draw_border_end", "todev_draw_obs", "todev_draw_obs_end", "todev_edgecmd", "todev_get_commondata", "todev_gethash", "todev_lat_up_ack", "todev_mow_task", "todev_one_touch_leave_pile", "todev_opt_border_info_ack", "todev_opt_line_up_ack", "todev_opt_obs_info_ack", "todev_planjob_set", "todev_rechgcmd", "todev_reset_chg_pile", "todev_save_task", "todev_sustask", "todev_svg_msg", "todev_task_info_ack", "todev_taskctrl", "todev_taskctrl_ack", "todev_unable_time_set", "todev_work_report_cmd", "todev_work_report_update_cmd", "todev_zigzag_ack", "vision_ctrl", "zone_start_precent"]
    ALL_PLAN_TASK_FIELD_NUMBER: _ClassVar[int]
    APP_REQUEST_COVER_PATHS_FIELD_NUMBER: _ClassVar[int]
    BIDIRE_REQCONVER_PATH_FIELD_NUMBER: _ClassVar[int]
    BIDIRE_TASKID_FIELD_NUMBER: _ClassVar[int]
    COVER_PATH_UPLOAD_FIELD_NUMBER: _ClassVar[int]
    NAV_SYS_PARAM_CMD_FIELD_NUMBER: _ClassVar[int]
    PLAN_TASK_EXECUTE_FIELD_NUMBER: _ClassVar[int]
    PLAN_TASK_NAME_ID_FIELD_NUMBER: _ClassVar[int]
    SIMULATION_CMD_FIELD_NUMBER: _ClassVar[int]
    TOAPP_ALL_HASH_NAME_FIELD_NUMBER: _ClassVar[int]
    TOAPP_BP_FIELD_NUMBER: _ClassVar[int]
    TOAPP_BSTATE_FIELD_NUMBER: _ClassVar[int]
    TOAPP_CHGPILETO_FIELD_NUMBER: _ClassVar[int]
    TOAPP_COSTMAP_FIELD_NUMBER: _ClassVar[int]
    TOAPP_EDGE_POINTS_ACK_FIELD_NUMBER: _ClassVar[int]
    TOAPP_EDGE_POINTS_FIELD_NUMBER: _ClassVar[int]
    TOAPP_GETHASH_ACK_FIELD_NUMBER: _ClassVar[int]
    TOAPP_GET_COMMONDATA_ACK_FIELD_NUMBER: _ClassVar[int]
    TOAPP_LAT_UP_FIELD_NUMBER: _ClassVar[int]
    TOAPP_MANUAL_ELEMENT_FIELD_NUMBER: _ClassVar[int]
    TOAPP_MAP_NAME_MSG_FIELD_NUMBER: _ClassVar[int]
    TOAPP_OPT_BORDER_INFO_FIELD_NUMBER: _ClassVar[int]
    TOAPP_OPT_LINE_UP_FIELD_NUMBER: _ClassVar[int]
    TOAPP_OPT_OBS_INFO_FIELD_NUMBER: _ClassVar[int]
    TOAPP_POS_UP_FIELD_NUMBER: _ClassVar[int]
    TOAPP_SVG_MSG_FIELD_NUMBER: _ClassVar[int]
    TOAPP_TASK_INFO_FIELD_NUMBER: _ClassVar[int]
    TOAPP_WORK_REPORT_ACK_FIELD_NUMBER: _ClassVar[int]
    TOAPP_WORK_REPORT_UPDATE_ACK_FIELD_NUMBER: _ClassVar[int]
    TOAPP_WORK_REPORT_UPLOAD_FIELD_NUMBER: _ClassVar[int]
    TOAPP_ZIGZAG_FIELD_NUMBER: _ClassVar[int]
    TODEV_CANCEL_DRAW_CMD_FIELD_NUMBER: _ClassVar[int]
    TODEV_CANCEL_SUSCMD_FIELD_NUMBER: _ClassVar[int]
    TODEV_CHL_LINE_DATA_FIELD_NUMBER: _ClassVar[int]
    TODEV_CHL_LINE_END_FIELD_NUMBER: _ClassVar[int]
    TODEV_CHL_LINE_FIELD_NUMBER: _ClassVar[int]
    TODEV_DRAW_BORDER_END_FIELD_NUMBER: _ClassVar[int]
    TODEV_DRAW_BORDER_FIELD_NUMBER: _ClassVar[int]
    TODEV_DRAW_OBS_END_FIELD_NUMBER: _ClassVar[int]
    TODEV_DRAW_OBS_FIELD_NUMBER: _ClassVar[int]
    TODEV_EDGECMD_FIELD_NUMBER: _ClassVar[int]
    TODEV_GETHASH_FIELD_NUMBER: _ClassVar[int]
    TODEV_GET_COMMONDATA_FIELD_NUMBER: _ClassVar[int]
    TODEV_LAT_UP_ACK_FIELD_NUMBER: _ClassVar[int]
    TODEV_MOW_TASK_FIELD_NUMBER: _ClassVar[int]
    TODEV_ONE_TOUCH_LEAVE_PILE_FIELD_NUMBER: _ClassVar[int]
    TODEV_OPT_BORDER_INFO_ACK_FIELD_NUMBER: _ClassVar[int]
    TODEV_OPT_LINE_UP_ACK_FIELD_NUMBER: _ClassVar[int]
    TODEV_OPT_OBS_INFO_ACK_FIELD_NUMBER: _ClassVar[int]
    TODEV_PLANJOB_SET_FIELD_NUMBER: _ClassVar[int]
    TODEV_RECHGCMD_FIELD_NUMBER: _ClassVar[int]
    TODEV_RESET_CHG_PILE_FIELD_NUMBER: _ClassVar[int]
    TODEV_SAVE_TASK_FIELD_NUMBER: _ClassVar[int]
    TODEV_SUSTASK_FIELD_NUMBER: _ClassVar[int]
    TODEV_SVG_MSG_FIELD_NUMBER: _ClassVar[int]
    TODEV_TASKCTRL_ACK_FIELD_NUMBER: _ClassVar[int]
    TODEV_TASKCTRL_FIELD_NUMBER: _ClassVar[int]
    TODEV_TASK_INFO_ACK_FIELD_NUMBER: _ClassVar[int]
    TODEV_UNABLE_TIME_SET_FIELD_NUMBER: _ClassVar[int]
    TODEV_WORK_REPORT_CMD_FIELD_NUMBER: _ClassVar[int]
    TODEV_WORK_REPORT_UPDATE_CMD_FIELD_NUMBER: _ClassVar[int]
    TODEV_ZIGZAG_ACK_FIELD_NUMBER: _ClassVar[int]
    VISION_CTRL_FIELD_NUMBER: _ClassVar[int]
    ZONE_START_PRECENT_FIELD_NUMBER: _ClassVar[int]
    all_plan_task: nav_get_all_plan_task
    app_request_cover_paths: app_request_cover_paths_t
    bidire_reqconver_path: NavReqCoverPath
    bidire_taskid: NavTaskIdRw
    cover_path_upload: cover_path_upload_t
    nav_sys_param_cmd: nav_sys_param_msg
    plan_task_execute: nav_plan_task_execute
    plan_task_name_id: plan_task_name_id_t
    simulation_cmd: SimulationCmdData
    toapp_all_hash_name: AppGetAllAreaHashName
    toapp_bp: NavTaskBreakPoint
    toapp_bstate: NavBorderState
    toapp_chgpileto: chargePileType
    toapp_costmap: costmap_t
    toapp_edge_points: NavEdgePoints
    toapp_edge_points_ack: NavEdgePointsAck
    toapp_get_commondata_ack: NavGetCommDataAck
    toapp_gethash_ack: NavGetHashListAck
    toapp_lat_up: NavLatLonUp
    toapp_manual_element: ManualElementMessage
    toapp_map_name_msg: NavMapNameMsg
    toapp_opt_border_info: NavOptiBorderInfo
    toapp_opt_line_up: NavOptLineUp
    toapp_opt_obs_info: NavOptObsInfo
    toapp_pos_up: NavPosUp
    toapp_svg_msg: svg_message_ack_t
    toapp_task_info: NavTaskInfo
    toapp_work_report_ack: WorkReportInfoAck
    toapp_work_report_update_ack: WorkReportUpdateAck
    toapp_work_report_upload: WorkReportInfoAck
    toapp_zigzag: NavUploadZigZagResult
    todev_cancel_draw_cmd: int
    todev_cancel_suscmd: int
    todev_chl_line: int
    todev_chl_line_data: NavCHlLineData
    todev_chl_line_end: int
    todev_draw_border: int
    todev_draw_border_end: int
    todev_draw_obs: int
    todev_draw_obs_end: int
    todev_edgecmd: int
    todev_get_commondata: NavGetCommData
    todev_gethash: NavGetHashList
    todev_lat_up_ack: int
    todev_mow_task: NavStartJob
    todev_one_touch_leave_pile: int
    todev_opt_border_info_ack: NavResFrame
    todev_opt_line_up_ack: NavResFrame
    todev_opt_obs_info_ack: NavResFrame
    todev_planjob_set: NavPlanJobSet
    todev_rechgcmd: int
    todev_reset_chg_pile: int
    todev_save_task: int
    todev_sustask: int
    todev_svg_msg: svg_message_ack_t
    todev_task_info_ack: NavResFrame
    todev_taskctrl: NavTaskCtrl
    todev_taskctrl_ack: NavTaskCtrlAck
    todev_unable_time_set: NavUnableTimeSet
    todev_work_report_cmd: WorkReportCmdData
    todev_work_report_update_cmd: WorkReportUpdateCmd
    todev_zigzag_ack: NavUploadZigZagResultAck
    vision_ctrl: vision_ctrl_msg
    zone_start_precent: zone_start_precent_t
    def __init__(self, toapp_lat_up: _Optional[_Union[NavLatLonUp, _Mapping]] = ..., toapp_pos_up: _Optional[_Union[NavPosUp, _Mapping]] = ..., todev_chl_line_data: _Optional[_Union[NavCHlLineData, _Mapping]] = ..., toapp_task_info: _Optional[_Union[NavTaskInfo, _Mapping]] = ..., toapp_opt_line_up: _Optional[_Union[NavOptLineUp, _Mapping]] = ..., toapp_opt_border_info: _Optional[_Union[NavOptiBorderInfo, _Mapping]] = ..., toapp_opt_obs_info: _Optional[_Union[NavOptObsInfo, _Mapping]] = ..., todev_task_info_ack: _Optional[_Union[NavResFrame, _Mapping]] = ..., todev_opt_border_info_ack: _Optional[_Union[NavResFrame, _Mapping]] = ..., todev_opt_obs_info_ack: _Optional[_Union[NavResFrame, _Mapping]] = ..., todev_opt_line_up_ack: _Optional[_Union[NavResFrame, _Mapping]] = ..., toapp_chgpileto: _Optional[_Union[chargePileType, _Mapping]] = ..., todev_sustask: _Optional[int] = ..., todev_rechgcmd: _Optional[int] = ..., todev_edgecmd: _Optional[int] = ..., todev_draw_border: _Optional[int] = ..., todev_draw_border_end: _Optional[int] = ..., todev_draw_obs: _Optional[int] = ..., todev_draw_obs_end: _Optional[int] = ..., todev_chl_line: _Optional[int] = ..., todev_chl_line_end: _Optional[int] = ..., todev_save_task: _Optional[int] = ..., todev_cancel_suscmd: _Optional[int] = ..., todev_reset_chg_pile: _Optional[int] = ..., todev_cancel_draw_cmd: _Optional[int] = ..., todev_one_touch_leave_pile: _Optional[int] = ..., todev_mow_task: _Optional[_Union[NavStartJob, _Mapping]] = ..., toapp_bstate: _Optional[_Union[NavBorderState, _Mapping]] = ..., todev_lat_up_ack: _Optional[int] = ..., todev_gethash: _Optional[_Union[NavGetHashList, _Mapping]] = ..., toapp_gethash_ack: _Optional[_Union[NavGetHashListAck, _Mapping]] = ..., todev_get_commondata: _Optional[_Union[NavGetCommData, _Mapping]] = ..., toapp_get_commondata_ack: _Optional[_Union[NavGetCommDataAck, _Mapping]] = ..., bidire_reqconver_path: _Optional[_Union[NavReqCoverPath, _Mapping]] = ..., toapp_zigzag: _Optional[_Union[NavUploadZigZagResult, _Mapping]] = ..., todev_zigzag_ack: _Optional[_Union[NavUploadZigZagResultAck, _Mapping]] = ..., todev_taskctrl: _Optional[_Union[NavTaskCtrl, _Mapping]] = ..., bidire_taskid: _Optional[_Union[NavTaskIdRw, _Mapping]] = ..., toapp_bp: _Optional[_Union[NavTaskBreakPoint, _Mapping]] = ..., todev_planjob_set: _Optional[_Union[NavPlanJobSet, _Mapping]] = ..., todev_unable_time_set: _Optional[_Union[NavUnableTimeSet, _Mapping]] = ..., simulation_cmd: _Optional[_Union[SimulationCmdData, _Mapping]] = ..., todev_work_report_update_cmd: _Optional[_Union[WorkReportUpdateCmd, _Mapping]] = ..., toapp_work_report_update_ack: _Optional[_Union[WorkReportUpdateAck, _Mapping]] = ..., todev_work_report_cmd: _Optional[_Union[WorkReportCmdData, _Mapping]] = ..., toapp_work_report_ack: _Optional[_Union[WorkReportInfoAck, _Mapping]] = ..., toapp_work_report_upload: _Optional[_Union[WorkReportInfoAck, _Mapping]] = ..., app_request_cover_paths: _Optional[_Union[app_request_cover_paths_t, _Mapping]] = ..., cover_path_upload: _Optional[_Union[cover_path_upload_t, _Mapping]] = ..., zone_start_precent: _Optional[_Union[zone_start_precent_t, _Mapping]] = ..., vision_ctrl: _Optional[_Union[vision_ctrl_msg, _Mapping]] = ..., nav_sys_param_cmd: _Optional[_Union[nav_sys_param_msg, _Mapping]] = ..., plan_task_execute: _Optional[_Union[nav_plan_task_execute, _Mapping]] = ..., toapp_costmap: _Optional[_Union[costmap_t, _Mapping]] = ..., plan_task_name_id: _Optional[_Union[plan_task_name_id_t, _Mapping]] = ..., all_plan_task: _Optional[_Union[nav_get_all_plan_task, _Mapping]] = ..., todev_taskctrl_ack: _Optional[_Union[NavTaskCtrlAck, _Mapping]] = ..., toapp_map_name_msg: _Optional[_Union[NavMapNameMsg, _Mapping]] = ..., todev_svg_msg: _Optional[_Union[svg_message_ack_t, _Mapping]] = ..., toapp_svg_msg: _Optional[_Union[svg_message_ack_t, _Mapping]] = ..., toapp_all_hash_name: _Optional[_Union[AppGetAllAreaHashName, _Mapping]] = ..., toapp_edge_points: _Optional[_Union[NavEdgePoints, _Mapping]] = ..., toapp_edge_points_ack: _Optional[_Union[NavEdgePointsAck, _Mapping]] = ..., toapp_manual_element: _Optional[_Union[ManualElementMessage, _Mapping]] = ...) -> None: ...

class NavBorderDataGet(_message.Message):
    __slots__ = ["borderLen", "currentFrame", "jobId"]
    BORDERLEN_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    borderLen: int
    currentFrame: int
    jobId: int
    def __init__(self, jobId: _Optional[int] = ..., currentFrame: _Optional[int] = ..., borderLen: _Optional[int] = ...) -> None: ...

class NavBorderDataGetAck(_message.Message):
    __slots__ = ["currentFrame", "jobId"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    jobId: int
    def __init__(self, jobId: _Optional[int] = ..., currentFrame: _Optional[int] = ...) -> None: ...

class NavBorderState(_message.Message):
    __slots__ = ["bdstate"]
    BDSTATE_FIELD_NUMBER: _ClassVar[int]
    bdstate: int
    def __init__(self, bdstate: _Optional[int] = ...) -> None: ...

class NavCHlLineData(_message.Message):
    __slots__ = ["channelLineLen", "currentFrame", "endJobRI", "startJobRI"]
    CHANNELLINELEN_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    ENDJOBRI_FIELD_NUMBER: _ClassVar[int]
    STARTJOBRI_FIELD_NUMBER: _ClassVar[int]
    channelLineLen: int
    currentFrame: int
    endJobRI: int
    startJobRI: int
    def __init__(self, startJobRI: _Optional[int] = ..., endJobRI: _Optional[int] = ..., currentFrame: _Optional[int] = ..., channelLineLen: _Optional[int] = ...) -> None: ...

class NavCHlLineDataAck(_message.Message):
    __slots__ = ["currentFrame", "endJobRI", "startJobRI"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    ENDJOBRI_FIELD_NUMBER: _ClassVar[int]
    STARTJOBRI_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    endJobRI: int
    startJobRI: int
    def __init__(self, startJobRI: _Optional[int] = ..., endJobRI: _Optional[int] = ..., currentFrame: _Optional[int] = ...) -> None: ...

class NavEdgePoints(_message.Message):
    __slots__ = ["Hash", "action", "currentFrame", "dataCouple", "result", "totalFrame", "type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATACOUPLE_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    Hash: int
    RESULT_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    action: int
    currentFrame: int
    dataCouple: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    result: int
    totalFrame: int
    type: int
    def __init__(self, result: _Optional[int] = ..., action: _Optional[int] = ..., type: _Optional[int] = ..., Hash: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataCouple: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ...) -> None: ...

class NavEdgePointsAck(_message.Message):
    __slots__ = ["Hash", "action", "currentFrame", "result", "totalFrame", "type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    Hash: int
    RESULT_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    action: int
    currentFrame: int
    result: int
    totalFrame: int
    type: int
    def __init__(self, result: _Optional[int] = ..., action: _Optional[int] = ..., type: _Optional[int] = ..., Hash: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ...) -> None: ...

class NavGetCommData(_message.Message):
    __slots__ = ["action", "currentFrame", "dataHash", "hash", "paternalHashA", "paternalHashB", "pver", "reserved", "subCmd", "totalFrame", "type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    PATERNALHASHA_FIELD_NUMBER: _ClassVar[int]
    PATERNALHASHB_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    action: int
    currentFrame: int
    dataHash: int
    hash: int
    paternalHashA: int
    paternalHashB: int
    pver: int
    reserved: str
    subCmd: int
    totalFrame: int
    type: int
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., action: _Optional[int] = ..., type: _Optional[int] = ..., hash: _Optional[int] = ..., paternalHashA: _Optional[int] = ..., paternalHashB: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataHash: _Optional[int] = ..., reserved: _Optional[str] = ...) -> None: ...

class NavGetCommDataAck(_message.Message):
    __slots__ = ["Hash", "action", "currentFrame", "dataCouple", "dataHash", "dataLen", "nameTime", "paternalHashA", "paternalHashB", "pver", "reserved", "result", "subCmd", "totalFrame", "type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATACOUPLE_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    DATALEN_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    Hash: int
    NAMETIME_FIELD_NUMBER: _ClassVar[int]
    PATERNALHASHA_FIELD_NUMBER: _ClassVar[int]
    PATERNALHASHB_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    action: int
    currentFrame: int
    dataCouple: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    dataHash: int
    dataLen: int
    nameTime: NavGetNameTime
    paternalHashA: int
    paternalHashB: int
    pver: int
    reserved: str
    result: int
    subCmd: int
    totalFrame: int
    type: int
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., result: _Optional[int] = ..., action: _Optional[int] = ..., type: _Optional[int] = ..., Hash: _Optional[int] = ..., paternalHashA: _Optional[int] = ..., paternalHashB: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataHash: _Optional[int] = ..., dataLen: _Optional[int] = ..., dataCouple: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ..., reserved: _Optional[str] = ..., nameTime: _Optional[_Union[NavGetNameTime, _Mapping]] = ...) -> None: ...

class NavGetHashList(_message.Message):
    __slots__ = ["currentFrame", "dataHash", "pver", "reserved", "subCmd", "totalFrame"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    dataHash: int
    pver: int
    reserved: str
    subCmd: int
    totalFrame: int
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataHash: _Optional[int] = ..., reserved: _Optional[str] = ...) -> None: ...

class NavGetHashListAck(_message.Message):
    __slots__ = ["currentFrame", "dataCouple", "dataHash", "hashLen", "pver", "reserved", "result", "subCmd", "totalFrame"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATACOUPLE_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    HASHLEN_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    dataCouple: _containers.RepeatedScalarFieldContainer[int]
    dataHash: int
    hashLen: int
    pver: int
    reserved: str
    result: int
    subCmd: int
    totalFrame: int
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataHash: _Optional[int] = ..., hashLen: _Optional[int] = ..., reserved: _Optional[str] = ..., result: _Optional[int] = ..., dataCouple: _Optional[_Iterable[int]] = ...) -> None: ...

class NavGetNameTime(_message.Message):
    __slots__ = ["create_time", "modify_time", "name"]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    MODIFY_TIME_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    create_time: int
    modify_time: int
    name: str
    def __init__(self, name: _Optional[str] = ..., create_time: _Optional[int] = ..., modify_time: _Optional[int] = ...) -> None: ...

class NavLatLonUp(_message.Message):
    __slots__ = ["lat", "lon"]
    LAT_FIELD_NUMBER: _ClassVar[int]
    LON_FIELD_NUMBER: _ClassVar[int]
    lat: float
    lon: float
    def __init__(self, lat: _Optional[float] = ..., lon: _Optional[float] = ...) -> None: ...

class NavMapNameMsg(_message.Message):
    __slots__ = ["deviceId", "hash", "name", "result", "rw"]
    DEVICEID_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    RW_FIELD_NUMBER: _ClassVar[int]
    deviceId: str
    hash: int
    name: str
    result: int
    rw: int
    def __init__(self, rw: _Optional[int] = ..., hash: _Optional[int] = ..., name: _Optional[str] = ..., result: _Optional[int] = ..., deviceId: _Optional[str] = ...) -> None: ...

class NavObstiBorderDataGet(_message.Message):
    __slots__ = ["currentFrame", "obstacleIndex", "obstaclesLen"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    OBSTACLEINDEX_FIELD_NUMBER: _ClassVar[int]
    OBSTACLESLEN_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    obstacleIndex: int
    obstaclesLen: int
    def __init__(self, obstacleIndex: _Optional[int] = ..., currentFrame: _Optional[int] = ..., obstaclesLen: _Optional[int] = ...) -> None: ...

class NavObstiBorderDataGetAck(_message.Message):
    __slots__ = ["currentFrame", "obstacleIndex"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    OBSTACLEINDEX_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    obstacleIndex: int
    def __init__(self, obstacleIndex: _Optional[int] = ..., currentFrame: _Optional[int] = ...) -> None: ...

class NavOptLineUp(_message.Message):
    __slots__ = ["allFrame", "channelDataLen", "currentFrame", "dc", "endJobRI", "startJobRI"]
    ALLFRAME_FIELD_NUMBER: _ClassVar[int]
    CHANNELDATALEN_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DC_FIELD_NUMBER: _ClassVar[int]
    ENDJOBRI_FIELD_NUMBER: _ClassVar[int]
    STARTJOBRI_FIELD_NUMBER: _ClassVar[int]
    allFrame: int
    channelDataLen: int
    currentFrame: int
    dc: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    endJobRI: int
    startJobRI: int
    def __init__(self, startJobRI: _Optional[int] = ..., endJobRI: _Optional[int] = ..., allFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., channelDataLen: _Optional[int] = ..., dc: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ...) -> None: ...

class NavOptObsInfo(_message.Message):
    __slots__ = ["allFrame", "currentFrame", "dc", "obstacleDataLen", "obstacleId"]
    ALLFRAME_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DC_FIELD_NUMBER: _ClassVar[int]
    OBSTACLEDATALEN_FIELD_NUMBER: _ClassVar[int]
    OBSTACLEID_FIELD_NUMBER: _ClassVar[int]
    allFrame: int
    currentFrame: int
    dc: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    obstacleDataLen: int
    obstacleId: int
    def __init__(self, obstacleId: _Optional[int] = ..., allFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., obstacleDataLen: _Optional[int] = ..., dc: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ...) -> None: ...

class NavOptiBorderInfo(_message.Message):
    __slots__ = ["allFrame", "borderDataLen", "currentFrame", "dc", "jobId"]
    ALLFRAME_FIELD_NUMBER: _ClassVar[int]
    BORDERDATALEN_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DC_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    allFrame: int
    borderDataLen: int
    currentFrame: int
    dc: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    jobId: int
    def __init__(self, jobId: _Optional[int] = ..., allFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., borderDataLen: _Optional[int] = ..., dc: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ...) -> None: ...

class NavPlanJobSet(_message.Message):
    __slots__ = ["PlanIndex", "area", "day", "deviceId", "edgeMode", "endDate", "endTime", "id", "jobId", "jobName", "knifeHeight", "model", "planId", "pver", "remained_seconds", "requiredTime", "reserved", "result", "routeAngle", "routeModel", "routeSpacing", "speed", "startDate", "startTime", "subCmd", "taskId", "taskName", "totalPlanNum", "towardIncludedAngle", "towardMode", "triggerType", "ultrasonicBarrier", "userId", "version", "week", "weeks", "workTime", "zoneHashs"]
    AREA_FIELD_NUMBER: _ClassVar[int]
    DAY_FIELD_NUMBER: _ClassVar[int]
    DEVICEID_FIELD_NUMBER: _ClassVar[int]
    EDGEMODE_FIELD_NUMBER: _ClassVar[int]
    ENDDATE_FIELD_NUMBER: _ClassVar[int]
    ENDTIME_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    JOBNAME_FIELD_NUMBER: _ClassVar[int]
    KNIFEHEIGHT_FIELD_NUMBER: _ClassVar[int]
    MODEL_FIELD_NUMBER: _ClassVar[int]
    PLANID_FIELD_NUMBER: _ClassVar[int]
    PLANINDEX_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    PlanIndex: int
    REMAINED_SECONDS_FIELD_NUMBER: _ClassVar[int]
    REQUIREDTIME_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    ROUTEANGLE_FIELD_NUMBER: _ClassVar[int]
    ROUTEMODEL_FIELD_NUMBER: _ClassVar[int]
    ROUTESPACING_FIELD_NUMBER: _ClassVar[int]
    SPEED_FIELD_NUMBER: _ClassVar[int]
    STARTDATE_FIELD_NUMBER: _ClassVar[int]
    STARTTIME_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TASKID_FIELD_NUMBER: _ClassVar[int]
    TASKNAME_FIELD_NUMBER: _ClassVar[int]
    TOTALPLANNUM_FIELD_NUMBER: _ClassVar[int]
    TOWARDINCLUDEDANGLE_FIELD_NUMBER: _ClassVar[int]
    TOWARDMODE_FIELD_NUMBER: _ClassVar[int]
    TRIGGERTYPE_FIELD_NUMBER: _ClassVar[int]
    ULTRASONICBARRIER_FIELD_NUMBER: _ClassVar[int]
    USERID_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    WEEKS_FIELD_NUMBER: _ClassVar[int]
    WEEK_FIELD_NUMBER: _ClassVar[int]
    WORKTIME_FIELD_NUMBER: _ClassVar[int]
    ZONEHASHS_FIELD_NUMBER: _ClassVar[int]
    area: int
    day: int
    deviceId: str
    edgeMode: int
    endDate: str
    endTime: str
    id: str
    jobId: str
    jobName: str
    knifeHeight: int
    model: int
    planId: str
    pver: int
    remained_seconds: int
    requiredTime: int
    reserved: str
    result: int
    routeAngle: int
    routeModel: int
    routeSpacing: int
    speed: float
    startDate: str
    startTime: str
    subCmd: int
    taskId: str
    taskName: str
    totalPlanNum: int
    towardIncludedAngle: int
    towardMode: int
    triggerType: int
    ultrasonicBarrier: int
    userId: str
    version: str
    week: int
    weeks: _containers.RepeatedScalarFieldContainer[int]
    workTime: int
    zoneHashs: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., area: _Optional[int] = ..., workTime: _Optional[int] = ..., version: _Optional[str] = ..., id: _Optional[str] = ..., userId: _Optional[str] = ..., deviceId: _Optional[str] = ..., planId: _Optional[str] = ..., taskId: _Optional[str] = ..., jobId: _Optional[str] = ..., startTime: _Optional[str] = ..., endTime: _Optional[str] = ..., week: _Optional[int] = ..., knifeHeight: _Optional[int] = ..., model: _Optional[int] = ..., edgeMode: _Optional[int] = ..., requiredTime: _Optional[int] = ..., routeAngle: _Optional[int] = ..., routeModel: _Optional[int] = ..., routeSpacing: _Optional[int] = ..., ultrasonicBarrier: _Optional[int] = ..., totalPlanNum: _Optional[int] = ..., PlanIndex: _Optional[int] = ..., result: _Optional[int] = ..., speed: _Optional[float] = ..., taskName: _Optional[str] = ..., jobName: _Optional[str] = ..., zoneHashs: _Optional[_Iterable[int]] = ..., reserved: _Optional[str] = ..., startDate: _Optional[str] = ..., endDate: _Optional[str] = ..., triggerType: _Optional[int] = ..., day: _Optional[int] = ..., weeks: _Optional[_Iterable[int]] = ..., remained_seconds: _Optional[int] = ..., towardMode: _Optional[int] = ..., towardIncludedAngle: _Optional[int] = ...) -> None: ...

class NavPosUp(_message.Message):
    __slots__ = ["age", "cHashId", "l2dfStars", "latStddev", "lonStddev", "posLevel", "posType", "stars", "status", "toward", "x", "y"]
    AGE_FIELD_NUMBER: _ClassVar[int]
    CHASHID_FIELD_NUMBER: _ClassVar[int]
    L2DFSTARS_FIELD_NUMBER: _ClassVar[int]
    LATSTDDEV_FIELD_NUMBER: _ClassVar[int]
    LONSTDDEV_FIELD_NUMBER: _ClassVar[int]
    POSLEVEL_FIELD_NUMBER: _ClassVar[int]
    POSTYPE_FIELD_NUMBER: _ClassVar[int]
    STARS_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    TOWARD_FIELD_NUMBER: _ClassVar[int]
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    age: float
    cHashId: int
    l2dfStars: int
    latStddev: float
    lonStddev: float
    posLevel: int
    posType: int
    stars: int
    status: int
    toward: int
    x: float
    y: float
    def __init__(self, x: _Optional[float] = ..., y: _Optional[float] = ..., status: _Optional[int] = ..., toward: _Optional[int] = ..., stars: _Optional[int] = ..., age: _Optional[float] = ..., latStddev: _Optional[float] = ..., lonStddev: _Optional[float] = ..., l2dfStars: _Optional[int] = ..., posType: _Optional[int] = ..., cHashId: _Optional[int] = ..., posLevel: _Optional[int] = ...) -> None: ...

class NavReqCoverPath(_message.Message):
    __slots__ = ["UltraWave", "channelMode", "channelWidth", "edgeMode", "jobId", "jobMode", "jobVer", "knifeHeight", "pathHash", "pver", "reserved", "result", "speed", "subCmd", "toward", "toward_included_angle", "toward_mode", "zoneHashs"]
    CHANNELMODE_FIELD_NUMBER: _ClassVar[int]
    CHANNELWIDTH_FIELD_NUMBER: _ClassVar[int]
    EDGEMODE_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    JOBMODE_FIELD_NUMBER: _ClassVar[int]
    JOBVER_FIELD_NUMBER: _ClassVar[int]
    KNIFEHEIGHT_FIELD_NUMBER: _ClassVar[int]
    PATHHASH_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SPEED_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TOWARD_FIELD_NUMBER: _ClassVar[int]
    TOWARD_INCLUDED_ANGLE_FIELD_NUMBER: _ClassVar[int]
    TOWARD_MODE_FIELD_NUMBER: _ClassVar[int]
    ULTRAWAVE_FIELD_NUMBER: _ClassVar[int]
    UltraWave: int
    ZONEHASHS_FIELD_NUMBER: _ClassVar[int]
    channelMode: int
    channelWidth: int
    edgeMode: int
    jobId: int
    jobMode: int
    jobVer: int
    knifeHeight: int
    pathHash: int
    pver: int
    reserved: str
    result: int
    speed: float
    subCmd: int
    toward: int
    toward_included_angle: int
    toward_mode: int
    zoneHashs: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, pver: _Optional[int] = ..., jobId: _Optional[int] = ..., jobVer: _Optional[int] = ..., jobMode: _Optional[int] = ..., subCmd: _Optional[int] = ..., edgeMode: _Optional[int] = ..., knifeHeight: _Optional[int] = ..., channelWidth: _Optional[int] = ..., UltraWave: _Optional[int] = ..., channelMode: _Optional[int] = ..., toward: _Optional[int] = ..., speed: _Optional[float] = ..., zoneHashs: _Optional[_Iterable[int]] = ..., pathHash: _Optional[int] = ..., reserved: _Optional[str] = ..., result: _Optional[int] = ..., toward_mode: _Optional[int] = ..., toward_included_angle: _Optional[int] = ...) -> None: ...

class NavResFrame(_message.Message):
    __slots__ = ["frameid"]
    FRAMEID_FIELD_NUMBER: _ClassVar[int]
    frameid: int
    def __init__(self, frameid: _Optional[int] = ...) -> None: ...

class NavStartJob(_message.Message):
    __slots__ = ["UltraWave", "channelMode", "channelWidth", "jobId", "jobMode", "jobVer", "knifeHeight", "rainTactics", "speed"]
    CHANNELMODE_FIELD_NUMBER: _ClassVar[int]
    CHANNELWIDTH_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    JOBMODE_FIELD_NUMBER: _ClassVar[int]
    JOBVER_FIELD_NUMBER: _ClassVar[int]
    KNIFEHEIGHT_FIELD_NUMBER: _ClassVar[int]
    RAINTACTICS_FIELD_NUMBER: _ClassVar[int]
    SPEED_FIELD_NUMBER: _ClassVar[int]
    ULTRAWAVE_FIELD_NUMBER: _ClassVar[int]
    UltraWave: int
    channelMode: int
    channelWidth: int
    jobId: int
    jobMode: int
    jobVer: int
    knifeHeight: int
    rainTactics: int
    speed: float
    def __init__(self, jobId: _Optional[int] = ..., jobVer: _Optional[int] = ..., jobMode: _Optional[int] = ..., rainTactics: _Optional[int] = ..., knifeHeight: _Optional[int] = ..., speed: _Optional[float] = ..., channelWidth: _Optional[int] = ..., UltraWave: _Optional[int] = ..., channelMode: _Optional[int] = ...) -> None: ...

class NavSysHashOverview(_message.Message):
    __slots__ = ["commonhashOverview", "pathHashOverview"]
    COMMONHASHOVERVIEW_FIELD_NUMBER: _ClassVar[int]
    PATHHASHOVERVIEW_FIELD_NUMBER: _ClassVar[int]
    commonhashOverview: int
    pathHashOverview: int
    def __init__(self, commonhashOverview: _Optional[int] = ..., pathHashOverview: _Optional[int] = ...) -> None: ...

class NavTaskBreakPoint(_message.Message):
    __slots__ = ["action", "flag", "toward", "x", "y", "zoneHash"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    FLAG_FIELD_NUMBER: _ClassVar[int]
    TOWARD_FIELD_NUMBER: _ClassVar[int]
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    ZONEHASH_FIELD_NUMBER: _ClassVar[int]
    action: int
    flag: int
    toward: int
    x: float
    y: float
    zoneHash: int
    def __init__(self, x: _Optional[float] = ..., y: _Optional[float] = ..., toward: _Optional[int] = ..., flag: _Optional[int] = ..., action: _Optional[int] = ..., zoneHash: _Optional[int] = ...) -> None: ...

class NavTaskCtrl(_message.Message):
    __slots__ = ["action", "reserved", "result", "type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    action: int
    reserved: str
    result: int
    type: int
    def __init__(self, type: _Optional[int] = ..., action: _Optional[int] = ..., result: _Optional[int] = ..., reserved: _Optional[str] = ...) -> None: ...

class NavTaskCtrlAck(_message.Message):
    __slots__ = ["action", "nav_state", "reserved", "result", "type"]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    NAV_STATE_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    action: int
    nav_state: int
    reserved: str
    result: int
    type: int
    def __init__(self, type: _Optional[int] = ..., action: _Optional[int] = ..., result: _Optional[int] = ..., nav_state: _Optional[int] = ..., reserved: _Optional[str] = ...) -> None: ...

class NavTaskIdRw(_message.Message):
    __slots__ = ["pver", "reserved", "result", "subCmd", "taskId", "taskName"]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TASKID_FIELD_NUMBER: _ClassVar[int]
    TASKNAME_FIELD_NUMBER: _ClassVar[int]
    pver: int
    reserved: str
    result: int
    subCmd: int
    taskId: str
    taskName: str
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., taskName: _Optional[str] = ..., taskId: _Optional[str] = ..., result: _Optional[int] = ..., reserved: _Optional[str] = ...) -> None: ...

class NavTaskInfo(_message.Message):
    __slots__ = ["allFrame", "area", "currentFrame", "dc", "pathlen", "time"]
    ALLFRAME_FIELD_NUMBER: _ClassVar[int]
    AREA_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DC_FIELD_NUMBER: _ClassVar[int]
    PATHLEN_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    allFrame: int
    area: int
    currentFrame: int
    dc: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    pathlen: int
    time: int
    def __init__(self, area: _Optional[int] = ..., time: _Optional[int] = ..., allFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., pathlen: _Optional[int] = ..., dc: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ...) -> None: ...

class NavTaskProgress(_message.Message):
    __slots__ = ["taskProgress"]
    TASKPROGRESS_FIELD_NUMBER: _ClassVar[int]
    taskProgress: int
    def __init__(self, taskProgress: _Optional[int] = ...) -> None: ...

class NavUnableTimeSet(_message.Message):
    __slots__ = ["deviceId", "reserved", "result", "subCmd", "trigger", "unableEndTime", "unableStartTime"]
    DEVICEID_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TRIGGER_FIELD_NUMBER: _ClassVar[int]
    UNABLEENDTIME_FIELD_NUMBER: _ClassVar[int]
    UNABLESTARTTIME_FIELD_NUMBER: _ClassVar[int]
    deviceId: str
    reserved: str
    result: int
    subCmd: int
    trigger: int
    unableEndTime: str
    unableStartTime: str
    def __init__(self, subCmd: _Optional[int] = ..., deviceId: _Optional[str] = ..., unableStartTime: _Optional[str] = ..., unableEndTime: _Optional[str] = ..., result: _Optional[int] = ..., reserved: _Optional[str] = ..., trigger: _Optional[int] = ...) -> None: ...

class NavUploadZigZagResult(_message.Message):
    __slots__ = ["area", "channelMode", "channelModeId", "currentFrame", "currentHash", "currentZone", "currentZonePathId", "currentZonePathNum", "dataCouple", "dataHash", "dataLen", "jobId", "jobVer", "pver", "reserved", "result", "subCmd", "time", "totalFrame", "totalZoneNum"]
    AREA_FIELD_NUMBER: _ClassVar[int]
    CHANNELMODEID_FIELD_NUMBER: _ClassVar[int]
    CHANNELMODE_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    CURRENTHASH_FIELD_NUMBER: _ClassVar[int]
    CURRENTZONEPATHID_FIELD_NUMBER: _ClassVar[int]
    CURRENTZONEPATHNUM_FIELD_NUMBER: _ClassVar[int]
    CURRENTZONE_FIELD_NUMBER: _ClassVar[int]
    DATACOUPLE_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    DATALEN_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    JOBVER_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TOTALZONENUM_FIELD_NUMBER: _ClassVar[int]
    area: int
    channelMode: int
    channelModeId: int
    currentFrame: int
    currentHash: int
    currentZone: int
    currentZonePathId: int
    currentZonePathNum: int
    dataCouple: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    dataHash: int
    dataLen: int
    jobId: int
    jobVer: int
    pver: int
    reserved: str
    result: int
    subCmd: int
    time: int
    totalFrame: int
    totalZoneNum: int
    def __init__(self, pver: _Optional[int] = ..., jobId: _Optional[int] = ..., jobVer: _Optional[int] = ..., result: _Optional[int] = ..., area: _Optional[int] = ..., time: _Optional[int] = ..., totalZoneNum: _Optional[int] = ..., currentZonePathNum: _Optional[int] = ..., currentZonePathId: _Optional[int] = ..., currentZone: _Optional[int] = ..., currentHash: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., channelMode: _Optional[int] = ..., channelModeId: _Optional[int] = ..., dataHash: _Optional[int] = ..., dataLen: _Optional[int] = ..., reserved: _Optional[str] = ..., dataCouple: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ..., subCmd: _Optional[int] = ...) -> None: ...

class NavUploadZigZagResultAck(_message.Message):
    __slots__ = ["currentFrame", "currentHash", "currentZone", "dataHash", "pver", "reserved", "subCmd", "totalFrame"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    CURRENTHASH_FIELD_NUMBER: _ClassVar[int]
    CURRENTZONE_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    currentHash: int
    currentZone: int
    dataHash: int
    pver: int
    reserved: str
    subCmd: int
    totalFrame: int
    def __init__(self, pver: _Optional[int] = ..., currentZone: _Optional[int] = ..., currentHash: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataHash: _Optional[int] = ..., reserved: _Optional[str] = ..., subCmd: _Optional[int] = ...) -> None: ...

class SimulationCmdData(_message.Message):
    __slots__ = ["param_id", "param_value", "subCmd"]
    PARAM_ID_FIELD_NUMBER: _ClassVar[int]
    PARAM_VALUE_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    param_id: int
    param_value: _containers.RepeatedScalarFieldContainer[int]
    subCmd: int
    def __init__(self, subCmd: _Optional[int] = ..., param_id: _Optional[int] = ..., param_value: _Optional[_Iterable[int]] = ...) -> None: ...

class WorkReportCmdData(_message.Message):
    __slots__ = ["getInfoNum", "subCmd"]
    GETINFONUM_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    getInfoNum: int
    subCmd: int
    def __init__(self, subCmd: _Optional[int] = ..., getInfoNum: _Optional[int] = ...) -> None: ...

class WorkReportInfoAck(_message.Message):
    __slots__ = ["current_ack_num", "end_work_time", "height_of_knife", "interrupt_flag", "job_content", "start_work_time", "total_ack_num", "work_ares", "work_progress", "work_result", "work_time_used", "work_type"]
    CURRENT_ACK_NUM_FIELD_NUMBER: _ClassVar[int]
    END_WORK_TIME_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_OF_KNIFE_FIELD_NUMBER: _ClassVar[int]
    INTERRUPT_FLAG_FIELD_NUMBER: _ClassVar[int]
    JOB_CONTENT_FIELD_NUMBER: _ClassVar[int]
    START_WORK_TIME_FIELD_NUMBER: _ClassVar[int]
    TOTAL_ACK_NUM_FIELD_NUMBER: _ClassVar[int]
    WORK_ARES_FIELD_NUMBER: _ClassVar[int]
    WORK_PROGRESS_FIELD_NUMBER: _ClassVar[int]
    WORK_RESULT_FIELD_NUMBER: _ClassVar[int]
    WORK_TIME_USED_FIELD_NUMBER: _ClassVar[int]
    WORK_TYPE_FIELD_NUMBER: _ClassVar[int]
    current_ack_num: int
    end_work_time: int
    height_of_knife: int
    interrupt_flag: bool
    job_content: int
    start_work_time: int
    total_ack_num: int
    work_ares: float
    work_progress: int
    work_result: int
    work_time_used: int
    work_type: int
    def __init__(self, interrupt_flag: bool = ..., start_work_time: _Optional[int] = ..., end_work_time: _Optional[int] = ..., work_time_used: _Optional[int] = ..., work_ares: _Optional[float] = ..., work_progress: _Optional[int] = ..., height_of_knife: _Optional[int] = ..., work_type: _Optional[int] = ..., work_result: _Optional[int] = ..., total_ack_num: _Optional[int] = ..., current_ack_num: _Optional[int] = ..., job_content: _Optional[int] = ...) -> None: ...

class WorkReportUpdateAck(_message.Message):
    __slots__ = ["info_num", "update_flag"]
    INFO_NUM_FIELD_NUMBER: _ClassVar[int]
    UPDATE_FLAG_FIELD_NUMBER: _ClassVar[int]
    info_num: int
    update_flag: bool
    def __init__(self, update_flag: bool = ..., info_num: _Optional[int] = ...) -> None: ...

class WorkReportUpdateCmd(_message.Message):
    __slots__ = ["subCmd"]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    subCmd: int
    def __init__(self, subCmd: _Optional[int] = ...) -> None: ...

class app_request_cover_paths_t(_message.Message):
    __slots__ = ["currentFrame", "dataHash", "hash_list", "pver", "reserved", "subCmd", "totalFrame", "transaction_id"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    HASH_LIST_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_ID_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    dataHash: int
    hash_list: _containers.RepeatedScalarFieldContainer[int]
    pver: int
    reserved: _containers.RepeatedScalarFieldContainer[int]
    subCmd: int
    totalFrame: int
    transaction_id: int
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataHash: _Optional[int] = ..., transaction_id: _Optional[int] = ..., reserved: _Optional[_Iterable[int]] = ..., hash_list: _Optional[_Iterable[int]] = ...) -> None: ...

class area_hash_name(_message.Message):
    __slots__ = ["hash", "name"]
    HASH_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    hash: int
    name: str
    def __init__(self, hash: _Optional[int] = ..., name: _Optional[str] = ...) -> None: ...

class chargePileType(_message.Message):
    __slots__ = ["toward", "x", "y"]
    TOWARD_FIELD_NUMBER: _ClassVar[int]
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    toward: int
    x: float
    y: float
    def __init__(self, toward: _Optional[int] = ..., x: _Optional[float] = ..., y: _Optional[float] = ...) -> None: ...

class costmap_t(_message.Message):
    __slots__ = ["center_x", "center_y", "costmap", "height", "res", "width", "yaw"]
    CENTER_X_FIELD_NUMBER: _ClassVar[int]
    CENTER_Y_FIELD_NUMBER: _ClassVar[int]
    COSTMAP_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    RES_FIELD_NUMBER: _ClassVar[int]
    WIDTH_FIELD_NUMBER: _ClassVar[int]
    YAW_FIELD_NUMBER: _ClassVar[int]
    center_x: float
    center_y: float
    costmap: _containers.RepeatedScalarFieldContainer[int]
    height: int
    res: float
    width: int
    yaw: float
    def __init__(self, width: _Optional[int] = ..., height: _Optional[int] = ..., center_x: _Optional[float] = ..., center_y: _Optional[float] = ..., yaw: _Optional[float] = ..., res: _Optional[float] = ..., costmap: _Optional[_Iterable[int]] = ...) -> None: ...

class cover_path_packet_t(_message.Message):
    __slots__ = ["dataCouple", "path_cur", "path_hash", "path_total", "path_type", "zone_hash"]
    DATACOUPLE_FIELD_NUMBER: _ClassVar[int]
    PATH_CUR_FIELD_NUMBER: _ClassVar[int]
    PATH_HASH_FIELD_NUMBER: _ClassVar[int]
    PATH_TOTAL_FIELD_NUMBER: _ClassVar[int]
    PATH_TYPE_FIELD_NUMBER: _ClassVar[int]
    ZONE_HASH_FIELD_NUMBER: _ClassVar[int]
    dataCouple: _containers.RepeatedCompositeFieldContainer[_common_pb2.CommDataCouple]
    path_cur: int
    path_hash: int
    path_total: int
    path_type: int
    zone_hash: int
    def __init__(self, path_hash: _Optional[int] = ..., path_type: _Optional[int] = ..., path_total: _Optional[int] = ..., path_cur: _Optional[int] = ..., zone_hash: _Optional[int] = ..., dataCouple: _Optional[_Iterable[_Union[_common_pb2.CommDataCouple, _Mapping]]] = ...) -> None: ...

class cover_path_upload_t(_message.Message):
    __slots__ = ["area", "currentFrame", "dataHash", "dataLen", "path_packets", "pver", "reserved", "result", "subCmd", "time", "totalFrame", "total_path_num", "transaction_id", "vaild_path_num"]
    AREA_FIELD_NUMBER: _ClassVar[int]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    DATALEN_FIELD_NUMBER: _ClassVar[int]
    PATH_PACKETS_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESERVED_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TOTAL_PATH_NUM_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_ID_FIELD_NUMBER: _ClassVar[int]
    VAILD_PATH_NUM_FIELD_NUMBER: _ClassVar[int]
    area: int
    currentFrame: int
    dataHash: int
    dataLen: int
    path_packets: _containers.RepeatedCompositeFieldContainer[cover_path_packet_t]
    pver: int
    reserved: _containers.RepeatedScalarFieldContainer[int]
    result: int
    subCmd: int
    time: int
    totalFrame: int
    total_path_num: int
    transaction_id: int
    vaild_path_num: int
    def __init__(self, pver: _Optional[int] = ..., result: _Optional[int] = ..., subCmd: _Optional[int] = ..., area: _Optional[int] = ..., time: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., total_path_num: _Optional[int] = ..., vaild_path_num: _Optional[int] = ..., dataHash: _Optional[int] = ..., transaction_id: _Optional[int] = ..., reserved: _Optional[_Iterable[int]] = ..., dataLen: _Optional[int] = ..., path_packets: _Optional[_Iterable[_Union[cover_path_packet_t, _Mapping]]] = ...) -> None: ...

class nav_get_all_plan_task(_message.Message):
    __slots__ = ["tasks"]
    TASKS_FIELD_NUMBER: _ClassVar[int]
    tasks: _containers.RepeatedCompositeFieldContainer[plan_task_name_id_t]
    def __init__(self, tasks: _Optional[_Iterable[_Union[plan_task_name_id_t, _Mapping]]] = ...) -> None: ...

class nav_plan_task_execute(_message.Message):
    __slots__ = ["id", "name", "result", "subCmd"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    result: int
    subCmd: int
    def __init__(self, subCmd: _Optional[int] = ..., id: _Optional[str] = ..., name: _Optional[str] = ..., result: _Optional[int] = ...) -> None: ...

class nav_sys_param_msg(_message.Message):
    __slots__ = ["context", "id", "rw"]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    RW_FIELD_NUMBER: _ClassVar[int]
    context: int
    id: int
    rw: int
    def __init__(self, rw: _Optional[int] = ..., id: _Optional[int] = ..., context: _Optional[int] = ...) -> None: ...

class plan_task_name_id_t(_message.Message):
    __slots__ = ["id", "name"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ...) -> None: ...

class svg_message_ack_t(_message.Message):
    __slots__ = ["currentFrame", "dataHash", "paternalHashA", "pver", "result", "subCmd", "svg_message", "totalFrame", "type"]
    CURRENTFRAME_FIELD_NUMBER: _ClassVar[int]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    PATERNALHASHA_FIELD_NUMBER: _ClassVar[int]
    PVER_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    SUBCMD_FIELD_NUMBER: _ClassVar[int]
    SVG_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    TOTALFRAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    currentFrame: int
    dataHash: int
    paternalHashA: int
    pver: int
    result: int
    subCmd: int
    svg_message: svg_message_t
    totalFrame: int
    type: int
    def __init__(self, pver: _Optional[int] = ..., subCmd: _Optional[int] = ..., totalFrame: _Optional[int] = ..., currentFrame: _Optional[int] = ..., dataHash: _Optional[int] = ..., paternalHashA: _Optional[int] = ..., type: _Optional[int] = ..., result: _Optional[int] = ..., svg_message: _Optional[_Union[svg_message_t, _Mapping]] = ...) -> None: ...

class svg_message_t(_message.Message):
    __slots__ = ["base_height_m", "base_height_pix", "base_width_m", "base_width_pix", "data_count", "hide_svg", "name_count", "rotate", "scale", "svg_file_data", "svg_file_name", "x_move", "y_move"]
    BASE_HEIGHT_M_FIELD_NUMBER: _ClassVar[int]
    BASE_HEIGHT_PIX_FIELD_NUMBER: _ClassVar[int]
    BASE_WIDTH_M_FIELD_NUMBER: _ClassVar[int]
    BASE_WIDTH_PIX_FIELD_NUMBER: _ClassVar[int]
    DATA_COUNT_FIELD_NUMBER: _ClassVar[int]
    HIDE_SVG_FIELD_NUMBER: _ClassVar[int]
    NAME_COUNT_FIELD_NUMBER: _ClassVar[int]
    ROTATE_FIELD_NUMBER: _ClassVar[int]
    SCALE_FIELD_NUMBER: _ClassVar[int]
    SVG_FILE_DATA_FIELD_NUMBER: _ClassVar[int]
    SVG_FILE_NAME_FIELD_NUMBER: _ClassVar[int]
    X_MOVE_FIELD_NUMBER: _ClassVar[int]
    Y_MOVE_FIELD_NUMBER: _ClassVar[int]
    base_height_m: float
    base_height_pix: int
    base_width_m: float
    base_width_pix: int
    data_count: int
    hide_svg: bool
    name_count: int
    rotate: float
    scale: float
    svg_file_data: str
    svg_file_name: str
    x_move: float
    y_move: float
    def __init__(self, x_move: _Optional[float] = ..., y_move: _Optional[float] = ..., scale: _Optional[float] = ..., rotate: _Optional[float] = ..., base_width_m: _Optional[float] = ..., base_height_m: _Optional[float] = ..., base_width_pix: _Optional[int] = ..., base_height_pix: _Optional[int] = ..., svg_file_name: _Optional[str] = ..., svg_file_data: _Optional[str] = ..., name_count: _Optional[int] = ..., data_count: _Optional[int] = ..., hide_svg: bool = ...) -> None: ...

class vision_ctrl_msg(_message.Message):
    __slots__ = ["cmd", "type"]
    CMD_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    cmd: int
    type: int
    def __init__(self, type: _Optional[int] = ..., cmd: _Optional[int] = ...) -> None: ...

class zone_start_precent_t(_message.Message):
    __slots__ = ["dataHash", "index", "x", "y"]
    DATAHASH_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    dataHash: int
    index: int
    x: float
    y: float
    def __init__(self, dataHash: _Optional[int] = ..., x: _Optional[float] = ..., y: _Optional[float] = ..., index: _Optional[int] = ...) -> None: ...



from asyncio import sleep
from io import BytesIO
import itertools
import json
import logging
import queue
import sys
import time

from bleak import BleakClient
from jsonic.serializable import serialize

from pymammotion.aliyun.tmp_constant import tmp_constant
from pymammotion.bluetooth.const import UUID_WRITE_CHARACTERISTIC
from pymammotion.bluetooth.data.framectrldata import FrameCtrlData
from pymammotion.bluetooth.data.notifydata import BlufiNotifyData
from pymammotion.bluetooth.model.atomic_integer import AtomicInteger
from pymammotion.data.model.execute_boarder import ExecuteBorder
from pymammotion.proto import DevNet, DrvDevInfoReq, LubaMsg, MsgAttr, MsgCmdType, MsgDevice
from pymammotion.utility.constant.device_constant import bleOrderCmd

_LOGGER = logging.getLogger(__name__)

CRC_TB = [
    0x0000,
    0x1021,
    0x2042,
    0x3063,
    0x4084,
    0x50A5,
    0x60C6,
    0x70E7,
    0x8108,
    0x9129,
    0xA14A,
    0xB16B,
    0xC18C,
    0xD1AD,
    0xE1CE,
    0xF1EF,
    0x1231,
    0x0210,
    0x3273,
    0x2252,
    0x52B5,
    0x4294,
    0x72F7,
    0x62D6,
    0x9339,
    0x8318,
    0xB37B,
    0xA35A,
    0xD3BD,
    0xC39C,
    0xF3FF,
    0xE3DE,
    0x2462,
    0x3443,
    0x0420,
    0x1401,
    0x64E6,
    0x74C7,
    0x44A4,
    0x5485,
    0xA56A,
    0xB54B,
    0x8528,
    0x9509,
    0xE5EE,
    0xF5CF,
    0xC5AC,
    0xD58D,
    0x3653,
    0x2672,
    0x1611,
    0x0630,
    0x76D7,
    0x66F6,
    0x5695,
    0x46B4,
    0xB75B,
    0xA77A,
    0x9719,
    0x8738,
    0xF7DF,
    0xE7FE,
    0xD79D,
    0xC7BC,
    0x48C4,
    0x58E5,
    0x6886,
    0x78A7,
    0x0840,
    0x1861,
    0x2802,
    0x3823,
    0xC9CC,
    0xD9ED,
    0xE98E,
    0xF9AF,
    0x8948,
    0x9969,
    0xA90A,
    0xB92B,
    0x5AF5,
    0x4AD4,
    0x7AB7,
    0x6A96,
    0x1A71,
    0x0A50,
    0x3A33,
    0x2A12,
    0xDBFD,
    0xCBDC,
    0xFBBF,
    0xEB9E,
    0x9B79,
    0x8B58,
    0xBB3B,
    0xAB1A,
    0x6CA6,
    0x7C87,
    0x4CE4,
    0x5CC5,
    0x2C22,
    0x3C03,
    0x0C60,
    0x1C41,
    0xEDAE,
    0xFD8F,
    0xCDEC,
    0xDDCD,
    0xAD2A,
    0xBD0B,
    0x8D68,
    0x9D49,
    0x7E97,
    0x6EB6,
    0x5ED5,
    0x4EF4,
    0x3E13,
    0x2E32,
    0x1E51,
    0x0E70,
    0xFF9F,
    0xEFBE,
    0xDFDD,
    0xCFFC,
    0xBF1B,
    0xAF3A,
    0x9F59,
    0x8F78,
    0x9188,
    0x81A9,
    0xB1CA,
    0xA1EB,
    0xD10C,
    0xC12D,
    0xF14E,
    0xE16F,
    0x1080,
    0x00A1,
    0x30C2,
    0x20E3,
    0x5004,
    0x4025,
    0x7046,
    0x6067,
    0x83B9,
    0x9398,
    0xA3FB,
    0xB3DA,
    0xC33D,
    0xD31C,
    0xE37F,
    0xF35E,
    0x02B1,
    0x1290,
    0x22F3,
    0x32D2,
    0x4235,
    0x5214,
    0x6277,
    0x7256,
    0xB5EA,
    0xA5CB,
    0x95A8,
    0x8589,
    0xF56E,
    0xE54F,
    0xD52C,
    0xC50D,
    0x34E2,
    0x24C3,
    0x14A0,
    0x0481,
    0x7466,
    0x6447,
    0x5424,
    0x4405,
    0xA7DB,
    0xB7FA,
    0x8799,
    0x97B8,
    0xE75F,
    0xF77E,
    0xC71D,
    0xD73C,
    0x26D3,
    0x36F2,
    0x0691,
    0x16B0,
    0x6657,
    0x7676,
    0x4615,
    0x5634,
    0xD94C,
    0xC96D,
    0xF90E,
    0xE92F,
    0x99C8,
    0x89E9,
    0xB98A,
    0xA9AB,
    0x5844,
    0x4865,
    0x7806,
    0x6827,
    0x18C0,
    0x08E1,
    0x3882,
    0x28A3,
    0xCB7D,
    0xDB5C,
    0xEB3F,
    0xFB1E,
    0x8BF9,
    0x9BD8,
    0xABBB,
    0xBB9A,
    0x4A75,
    0x5A54,
    0x6A37,
    0x7A16,
    0x0AF1,
    0x1AD0,
    0x2AB3,
    0x3A92,
    0xFD2E,
    0xED0F,
    0xDD6C,
    0xCD4D,
    0xBDAA,
    0xAD8B,
    0x9DE8,
    0x8DC9,
    0x7C26,
    0x6C07,
    0x5C64,
    0x4C45,
    0x3CA2,
    0x2C83,
    0x1CE0,
    0x0CC1,
    0xEF1F,
    0xFF3E,
    0xCF5D,
    0xDF7C,
    0xAF9B,
    0xBFBA,
    0x8FD9,
    0x9FF8,
    0x6E17,
    0x7E36,
    0x4E55,
    0x5E74,
    0x2E93,
    0x3EB2,
    0x0ED1,
    0x1EF0,
]


class BleMessage:
    """Class for sending and recieving messages from Luba"""

    AES_TRANSFORMATION = "AES/CFB/NoPadding"
    DEFAULT_PACKAGE_LENGTH = 20
    DH_G = "2"
    DH_P = "cf5cf5c38419a724957ff5dd323b9c45c3cdd261eb740f69aa94b8bb1a5c96409153bd76b24222d03274e4725a5406092e9e82e9135c643cae98132b0d95f7d65347c68afc1e677da90e51bbab5f5cf429c291b4ba39c6b2dc5e8c7231e46aa7728e87664532cdf547be20c9a3fa8342be6e34371a27c06f7dc0edddd2f86373"
    MIN_PACKAGE_LENGTH = 20
    NEG_SECURITY_SET_ALL_DATA = 1
    NEG_SECURITY_SET_TOTAL_LENGTH = 0
    PACKAGE_HEADER_LENGTH = 4
    mPrintDebug = False
    mWriteTimeout = -1
    mPackageLengthLimit = -1
    mBlufiMTU = -1
    mEncrypted = False
    mChecksum = False
    mRequireAck = False
    mConnectState = 0

    def __init__(self, client: BleakClient) -> None:
        self.client = client
        self.mSendSequence = AtomicInteger(-1)
        self.mReadSequence = AtomicInteger(-1)
        self.mAck = queue.Queue()
        self.notification = BlufiNotifyData()

    async def get_task(self) -> None:
        hash_map = {"pver": 1, "subCmd": 2, "result": 0}
        await self.post_custom_data(self.get_json_string(bleOrderCmd.task, hash_map))

    async def send_ble_alive(self) -> None:
        hash_map = {"ctrl": 1}
        await self.post_custom_data(self.get_json_string(bleOrderCmd.bleAlive, hash_map))

    def get_json_string(self, cmd: int, hash_map: dict[str, int]) -> str:
        jSONObject = {}
        try:
            jSONObject["cmd"] = cmd
            jSONObject[tmp_constant.REQUEST_ID] = int(time.time())
            jSONObject2 = {}
            for key, value in hash_map.items():
                jSONObject2[key] = value
            jSONObject["params"] = jSONObject2
            return json.dumps(jSONObject)
        except Exception as e:
            print(e)
            return ""

    def clear_notification(self) -> None:
        self.notification = None
        self.notification = BlufiNotifyData()

    # async def get_device_info(self):
    #     await self.postCustomData(self.getJsonString(bleOrderCmd.getDeviceInfo))

    async def send_device_info(self) -> None:
        """Currently not called"""
        luba_msg = LubaMsg(
            msgtype=MsgCmdType.ESP,
            sender=MsgDevice.DEV_MOBILEAPP,
            rcver=MsgDevice.DEV_COMM_ESP,
            msgattr=MsgAttr.REQ,
            seqs=1,
            version=1,
            subtype=1,
            net=DevNet(todev_ble_sync=1, todev_devinfo_req=DrvDevInfoReq()),
        )
        byte_arr = luba_msg.SerializeToString()
        await self.post_custom_data_bytes(byte_arr)

    async def requestDeviceStatus(self) -> None:
        request = False
        type = self.getTypeValue(0, 5)
        try:
            request = await self.post(BleMessage.mEncrypted, BleMessage.mChecksum, False, type, None)
            # _LOGGER.debug(request)
        except Exception as err:
            # Log.w(TAG, "post requestDeviceStatus interrupted")
            request = False
            _LOGGER.error(err)

        # if not request:
        #     onStatusResponse(BlufiCallback.CODE_WRITE_DATA_FAILED, null)

    async def requestDeviceVersion(self) -> None:
        request = False
        type = self.getTypeValue(0, 7)
        try:
            request = await self.post(BleMessage.mEncrypted, BleMessage.mChecksum, False, type, None)
            # _LOGGER.debug(request)
        except Exception as err:
            # Log.w(TAG, "post requestDeviceStatus interrupted")
            request = False
            _LOGGER.error(err)

    async def sendBorderPackage(self, executeBorder: ExecuteBorder) -> None:
        await self.post_custom_data(serialize(executeBorder))

    async def gatt_write(self, data: bytes) -> None:
        await self.client.write_gatt_char(UUID_WRITE_CHARACTERISTIC, data, True)

    def parseNotification(self, response: bytearray):
        """Parse notification data from BLE device."""
        if response is None:
            # Log.w(TAG, "parseNotification null data");
            return -1

        # if (this.mPrintDebug):
        #     Log.d(TAG, "parseNotification Notification= " + Arrays.toString(response));
        # }
        if len(response) < 4:
            _LOGGER.debug("parseNotification data length less than 4")
            return -2

        sequence = int(response[2])  # toInt
        current_sequence = self.mReadSequence.get() & 255
        if sequence == current_sequence:
            # _LOGGER.debug(f"Received bluetooth data 1: {response.hex()}, object: {self}")
            return 2

        # Compare with the second counter, mod 255
        if sequence != (self.mReadSequence.increment_and_get() & 255):
            _LOGGER.debug(
                "parseNotification read sequence wrong %s %s",
                sequence,
                self.mReadSequence.get(),
            )
            # Set the value for mReadSequence manually
            self.mReadSequence.set(sequence)

        # LogUtil.m7773e(self.mGatt.getDevice().getName() + "打印丢包率", self.mReadSequence_2 + "/" + self.mReadSequence_1);
        pkt_type = int(response[0])  # toInt
        pkgType = self._getPackageType(pkt_type)
        subType = self._getSubType(pkt_type)
        self.notification.setType(pkt_type)
        self.notification.setPkgType(pkgType)
        self.notification.setSubType(subType)
        frameCtrl = int(response[1])  # toInt
        # _LOGGER.debug("frame ctrl")
        # _LOGGER.debug(frameCtrl)
        # _LOGGER.debug(response)
        # _LOGGER.debug(f"pktType {pkt_type} pkgType {pkgType} subType {subType}")
        self.notification.setFrameCtrl(frameCtrl)
        frameCtrlData = FrameCtrlData(frameCtrl)
        dataLen = int(response[3])  # toInt specifies length of data

        try:
            dataBytes = response[4 : 4 + dataLen]
            if frameCtrlData.isEncrypted():
                _LOGGER.debug("is encrypted")
            #     BlufiAES aes = new BlufiAES(self.mAESKey, AES_TRANSFORMATION, generateAESIV(sequence));
            #     dataBytes = aes.decrypt(dataBytes);
            # }
            if frameCtrlData.isChecksum():
                respChecksum1 = int(response[-1])
                respChecksum2 = int(response[-2])
                crc = self.calc_crc(self.calc_crc(0, bytes([sequence, dataLen])), dataBytes)
                calcChecksum1 = (crc >> 8) & 255
                calcChecksum2 = crc & 255

                if respChecksum1 != calcChecksum1 or respChecksum2 != calcChecksum2:
                    _LOGGER.debug(
                        f"expect checksum: {respChecksum1}, {respChecksum2}\n"
                        f"received checksum: {calcChecksum1}, {calcChecksum2}"
                    )
                    return -4

            data_offset = 2 if frameCtrlData.hasFrag() else 0

            self.notification.addData(dataBytes, data_offset)
            return 1 if frameCtrlData.hasFrag() else 0
        except Exception as e:
            _LOGGER.debug(e)
            return -100

    async def parseBlufiNotifyData(self, return_bytes: bool = False):
        pkgType = self.notification.getPkgType()
        subType = self.notification.getSubType()
        dataBytes = self.notification.getDataArray()
        if pkgType == 0:
            # never seem to get these..
            self._parseCtrlData(subType, dataBytes)
        if pkgType == 1:
            if return_bytes:
                return dataBytes
            return await self._parseDataData(subType, dataBytes)

    def _parseCtrlData(self, subType: int, data: bytes) -> None:
        pass
        # self._parseAck(data)

    async def _parseDataData(self, subType: int, data: bytes):
        #     if (subType == 0) {
        #         this.mSecurityCallback.onReceiveDevicePublicKey(data);
        #         return;
        #     }
        _LOGGER.debug(subType)
        match subType:
            #         case 15:
            #             parseWifiState(data);
            #             return;
            #         case 16:
            #             parseVersion(data);
            #             return;
            #         case 17:
            #             parseWifiScanList(data);
            #             return;
            #         case 18:
            #             int errCode = data.length > 0 ? 255 & data[0] : 255;
            #             onError(errCode);
            #             return;
            case 19:
                #             # com/agilexrobotics/utils/EspBleUtil$BlufiCallbackMain.smali
                # luba_msg = parse_custom_data(data)  # parse to protobuf message
                return data

    # private void parseCtrlData(int i, byte[] bArr) {
    #     if (i == 0) {
    #         parseAck(bArr);
    #     }
    # }

    # private void parseAck(byte[] bArr) {
    #     this.mAck.add(Integer.valueOf(bArr.length > 0 ? bArr[0] & 255 : 256));
    # }

    def getJsonString(self, cmd: int) -> str:
        jSONObject = {}
        try:
            jSONObject["cmd"] = cmd
            jSONObject[tmp_constant.REQUEST_ID] = int(time.time())
            return json.dumps(jSONObject)
        except Exception:
            return ""

    def current_milli_time(self):
        return round(time.time() * 1000)

    def _getPackageType(self, typeValue: int):
        return typeValue & 3

    def _getSubType(self, typeValue: int):
        return (typeValue & 252) >> 2

    def getTypeValue(self, type: int, subtype: int):
        return (subtype << 2) | type

    def receiveAck(self, expectAck: int) -> bool:
        try:
            ack = self.mAck.get()
            return ack == expectAck
        except Exception as err:
            _LOGGER.debug(err)
            return False

    def generate_send_sequence(self) -> int:
        return self.mSendSequence.increment_and_get() & 255

    async def post_custom_data_bytes(self, data: bytes) -> None:
        if data is None:
            return
        type_val = self.getTypeValue(1, 19)
        try:
            suc = await self.post(self.mEncrypted, self.mChecksum, self.mRequireAck, type_val, data)
            # int status = suc ? 0 : BlufiCallback.CODE_WRITE_DATA_FAILED
            # onPostCustomDataResult(status, data)
            # _LOGGER.debug(suc)
        except Exception as err:
            await self.client.disconnect()
            _LOGGER.debug(err)

    async def post_custom_data(self, data_str: str) -> None:
        data = data_str.encode()
        if data == None:
            return
        type_val = self.getTypeValue(1, 19)
        try:
            suc = await self.post(self.mEncrypted, self.mChecksum, self.mRequireAck, type_val, data)
            # int status = suc ? 0 : BlufiCallback.CODE_WRITE_DATA_FAILED
            # onPostCustomDataResult(status, data)
        except Exception as err:
            _LOGGER.debug(err)
            # we might be constantly connected and in a bad state
            self.mSendSequence = itertools.count()
            self.mReadSequence = itertools.count()
            await self.client.disconnect()

    async def post(
        self,
        encrypt: bool,
        checksum: bool,
        require_ack: bool,
        type_of: int,
        data: bytes,
    ) -> bool:
        if data is None:
            return await self.post_non_data(encrypt, checksum, require_ack, type_of)

        return await self.post_contains_data(encrypt, checksum, require_ack, type_of, data)

    async def post_non_data(self, encrypt: bool, checksum: bool, require_ack: bool, type_of: int) -> bool:
        sequence = self.generate_send_sequence()
        postBytes = self.getPostBytes(type_of, encrypt, checksum, require_ack, False, sequence, None)
        posted = await self.gatt_write(postBytes)
        return posted and (not require_ack or self.receiveAck(sequence))

    async def post_contains_data(
        self,
        encrypt: bool,
        checksum: bool,
        require_ack: bool,
        type_of: int,
        data: bytes,
    ) -> bool:
        chunk_size = 517  # self.client.mtu_size - 3

        chunks = list()
        for i in range(0, len(data), chunk_size):
            if i + chunk_size > len(data):
                chunks.append(data[i : len(data)])
            else:
                chunks.append(data[i : i + chunk_size])
        for index, chunk in enumerate(chunks):
            frag = index != len(chunks) - 1
            sequence = self.generate_send_sequence()
            postBytes = self.getPostBytes(type_of, encrypt, checksum, require_ack, frag, sequence, chunk)
            # _LOGGER.debug("sequence")
            # _LOGGER.debug(sequence)
            posted = await self.gatt_write(postBytes)
            if posted is not None:
                return False

            if not frag:
                return not require_ack or self.receiveAck(sequence)

            if require_ack and not self.receiveAck(sequence):
                return False

            _LOGGER.debug("sleeping 0.01")
            await sleep(0.01)
            if require_ack and not self.receiveAck(sequence):
                return False
            else:
                return True

    def getPostBytes(
        self,
        type: int,
        encrypt: bool,
        checksum: bool,
        require_ack: bool,
        hasFrag: bool,
        sequence: int,
        data: bytes | None,
    ) -> bytes:
        byteOS = BytesIO()
        dataLength = 0 if data == None else len(data)
        frameCtrl = FrameCtrlData.getFrameCTRLValue(encrypt, checksum, 0, require_ack, hasFrag)
        byteOS.write(type.to_bytes(1, sys.byteorder))
        byteOS.write(frameCtrl.to_bytes(1, sys.byteorder))
        byteOS.write(sequence.to_bytes(1, sys.byteorder))
        byteOS.write(dataLength.to_bytes(1, sys.byteorder))

        if data is not None:
            byteOS.write(data)

        _LOGGER.debug(byteOS.getvalue())
        return byteOS.getvalue()

    @staticmethod
    def calc_crc(initial: int, data: bytes | bytearray) -> int:
        """Calculate CRC value for given initial value and byte array.

        Args:
            initial: Initial CRC value
            data: Bytes to calculate CRC for

        Returns:
            Calculated CRC value (16-bit)

        Raises:
            TypeError: If data is not bytes or bytearray
            ValueError: If initial value is out of valid range

        """
        if not isinstance(data, (bytes, bytearray)):
            raise TypeError("Data must be bytes or bytearray")

        if not 0 <= initial <= 0xFFFF:
            raise ValueError("Initial value must be between 0 and 65535")

        try:
            crc = (~initial) & 0xFFFF

            for byte in data:
                crc = ((crc << 8) ^ CRC_TB[byte ^ (crc >> 8)]) & 0xFFFF

            return (~crc) & 0xFFFF

        except Exception as e:
            _LOGGER.error("Error calculating CRC: %s", str(e))
            raise



import asyncio
from collections.abc import Callable
from types import MethodType
from typing import Any
import weakref


class Event:
    def __init__(self) -> None:
        self.__eventhandlers: list[weakref.ReferenceType] = []

    def __iadd__(self, handler: Callable) -> "Event":
        if isinstance(handler, MethodType):
            # Instance method, use WeakMethod
            ref = weakref.WeakMethod(handler)
        else:
            # Function or static method, use weakref.ref
            ref = weakref.ref(handler)
        self.__eventhandlers.append(ref)
        return self

    def __isub__(self, handler: Callable) -> "Event":
        self.__eventhandlers = [ref for ref in self.__eventhandlers if ref() is not handler]
        return self

    async def __call__(self, *args: Any, **kwargs: Any) -> None:
        live_handlers = []
        for ref in self.__eventhandlers:
            func = ref()
            if func is not None:
                live_handlers.append(func(*args, **kwargs))
        await asyncio.gather(*live_handlers)

        # Clean up dead references
        self.__eventhandlers = [ref for ref in self.__eventhandlers if ref() is not None]

    def has_dead_handlers(self) -> bool:
        """Check if any handlers have been garbage collected."""
        return any(ref() is None for ref in self.__eventhandlers)


class MoveEvent:
    def __init__(self) -> None:
        self.OnMoveFinished = Event()

    async def MoveFinished(self) -> None:
        # This function will be executed once blufi finishes after a movement command and will
        # raise an event
        await self.OnMoveFinished()

    def AddSubscribersForMoveFinishedEvent(self, objMethod) -> None:
        self.OnMoveFinished += objMethod

    def RemoveSubscribersForMoveFinishedEvent(self, objMethod) -> None:
        self.OnMoveFinished -= objMethod


class BleNotificationEvent:
    def __init__(self) -> None:
        self.OnBleNotification = Event()

    async def BleNotification(self, data: bytearray) -> None:
        # This function will be executed when data is received.
        await self.OnBleNotification(data)

    def AddSubscribersForBleNotificationEvent(self, objMethod) -> None:
        self.OnBleNotification += objMethod

    def RemoveSubscribersForBleNotificationEvent(self, objMethod) -> None:
        self.OnBleNotification -= objMethod


class DataEvent:
    """Callbacks for data events."""

    def __init__(self) -> None:
        self.on_data_event = Event()

    async def data_event(self, data: Any) -> None:
        # This function will be executed when data is received.
        if data:
            await self.on_data_event(data)
        else:
            await self.on_data_event()

    def add_subscribers(self, obj_method) -> None:
        self.on_data_event += obj_method

    def remove_subscribers(self, obj_method) -> None:
        try:
            self.on_data_event -= obj_method
        except ValueError:
            """Subscription object no longer there."""



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pymammotion/proto/mctrl_ota.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!pymammotion/proto/mctrl_ota.proto\"+\n\x08uplink_t\x12\x0f\n\x07pkg_seq\x18\x01 \x01(\x05\x12\x0e\n\x06status\x18\x02 \x01(\x05\":\n\ndownlink_t\x12\r\n\x05\x66w_id\x18\x01 \x01(\x05\x12\x0f\n\x07pkg_seq\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x05\"p\n\x0e\x66wDownloadCtrl\x12\x1b\n\x06uplink\x18\x02 \x01(\x0b\x32\t.uplink_tH\x00\x12\x1f\n\x08\x64ownlink\x18\x03 \x01(\x0b\x32\x0b.downlink_tH\x00\x12\x0b\n\x03\x63md\x18\x01 \x01(\x05\x42\x13\n\x11SubFwDownloadCtrl\"k\n\x08\x62\x61seInfo\x12\x13\n\x0b\x64\x65v_version\x18\x01 \x01(\t\x12\x12\n\ndev_status\x18\x02 \x01(\x05\x12\x10\n\x08\x62\x61tt_val\x18\x03 \x01(\x05\x12\x13\n\x0binit_status\x18\x04 \x01(\x05\x12\x0f\n\x07is_tilt\x18\x05 \x01(\x05\"\\\n\x07otaInfo\x12\r\n\x05otaid\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x10\n\x08progress\x18\x03 \x01(\x05\x12\x0e\n\x06result\x18\x04 \x01(\x05\x12\x0f\n\x07message\x18\x05 \x01(\t\"%\n\ngetInfoReq\x12\x17\n\x04type\x18\x01 \x01(\x0e\x32\t.infoType\"q\n\ngetInfoRsp\x12\x19\n\x04\x62\x61se\x18\x03 \x01(\x0b\x32\t.baseInfoH\x00\x12\x17\n\x03ota\x18\x04 \x01(\x0b\x32\x08.otaInfoH\x00\x12\x0e\n\x06result\x18\x01 \x01(\x05\x12\x17\n\x04type\x18\x02 \x01(\x0e\x32\t.infoTypeB\x06\n\x04info\"v\n\nFotaInfo_t\x12\x14\n\x0cneed_ota_num\x18\x01 \x01(\x05\x12\x19\n\x11need_ota_img_size\x18\x02 \x01(\x05\x12\x11\n\tota_otype\x18\x03 \x01(\x05\x12\x0f\n\x07ota_oid\x18\x04 \x01(\t\x12\x13\n\x0bota_version\x18\x05 \x01(\t\"\x81\x01\n\rFotaSubInfo_t\x12\x18\n\x10sub_mod_ota_flag\x18\x01 \x01(\x05\x12\x12\n\nsub_mod_id\x18\x02 \x01(\x05\x12\x14\n\x0csub_img_size\x18\x03 \x01(\x05\x12\x17\n\x0fsub_mod_version\x18\x04 \x01(\t\x12\x13\n\x0bsub_img_url\x18\x05 \x01(\t\"\xe4\x01\n\x07MctlOta\x12)\n\x12todev_get_info_req\x18\x01 \x01(\x0b\x32\x0b.getInfoReqH\x00\x12)\n\x12toapp_get_info_rsp\x18\x02 \x01(\x0b\x32\x0b.getInfoRspH\x00\x12+\n\x10\x66w_download_ctrl\x18\x03 \x01(\x0b\x32\x0f.fwDownloadCtrlH\x00\x12 \n\tfota_info\x18\x04 \x01(\x0b\x32\x0b.FotaInfo_tH\x00\x12\'\n\rfota_sub_info\x18\x05 \x01(\x0b\x32\x0e.FotaSubInfo_tH\x00\x42\x0b\n\tSubOtaMsg*#\n\x08infoType\x12\x0b\n\x07IT_BASE\x10\x00\x12\n\n\x06IT_OTA\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pymammotion.proto.mctrl_ota_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INFOTYPE._serialized_start=1096
  _INFOTYPE._serialized_end=1131
  _UPLINK_T._serialized_start=37
  _UPLINK_T._serialized_end=80
  _DOWNLINK_T._serialized_start=82
  _DOWNLINK_T._serialized_end=140
  _FWDOWNLOADCTRL._serialized_start=142
  _FWDOWNLOADCTRL._serialized_end=254
  _BASEINFO._serialized_start=256
  _BASEINFO._serialized_end=363
  _OTAINFO._serialized_start=365
  _OTAINFO._serialized_end=457
  _GETINFOREQ._serialized_start=459
  _GETINFOREQ._serialized_end=496
  _GETINFORSP._serialized_start=498
  _GETINFORSP._serialized_end=611
  _FOTAINFO_T._serialized_start=613
  _FOTAINFO_T._serialized_end=731
  _FOTASUBINFO_T._serialized_start=734
  _FOTASUBINFO_T._serialized_end=863
  _MCTLOTA._serialized_start=866
  _MCTLOTA._serialized_end=1094
# @@protoc_insertion_point(module_scope)



import struct


class MurMurHashUtil:
    @staticmethod
    def get_unsigned_int(i):
        return i & 0xFFFFFFFF

    @staticmethod
    def hash(byte_arr: bytes):
        # Create a bytearray view with little endian order
        position = 0
        remaining_bytes = len(byte_arr)

        # Initial values
        remaining = remaining_bytes ^ 97
        j = 0

        # Process 8 bytes at a time
        while remaining_bytes >= 8:
            multiplier = 1540483477

            # First 4 bytes
            unsigned_int = struct.unpack_from("<I", byte_arr, position)[0]
            position += 4
            unsigned_int = (MurMurHashUtil.get_unsigned_int(unsigned_int) * multiplier) & 0xFFFFFFFF
            remaining = (
                ((remaining * multiplier) & 0xFFFFFFFF)
                ^ (((unsigned_int ^ (unsigned_int >> 24)) & 0xFFFFFFFF) * multiplier) & 0xFFFFFFFF
            ) & 0xFFFFFFFF

            # Next 4 bytes
            unsigned_int2 = struct.unpack_from("<I", byte_arr, position)[0]
            position += 4
            unsigned_int2 = (MurMurHashUtil.get_unsigned_int(unsigned_int2) * multiplier) & 0xFFFFFFFF
            j = (
                ((j * multiplier) & 0xFFFFFFFF)
                ^ ((((unsigned_int2 ^ (unsigned_int2 >> 24)) & 0xFFFFFFFF) * multiplier) & 0xFFFFFFFF)
            ) & 0xFFFFFFFF

            remaining_bytes -= 8

        # Process remaining 4 bytes if available
        if remaining_bytes >= 4:
            multiplier = 1540483477
            unsigned_int3 = struct.unpack_from("<I", byte_arr, position)[0]
            position += 4
            unsigned_int3 = (MurMurHashUtil.get_unsigned_int(unsigned_int3) * multiplier) & 0xFFFFFFFF
            remaining = (
                ((remaining * multiplier) & 0xFFFFFFFF)
                ^ ((((unsigned_int3 ^ (unsigned_int3 >> 24)) & 0xFFFFFFFF) * multiplier) & 0xFFFFFFFF)
            ) & 0xFFFFFFFF
            remaining_bytes -= 4

        # Process final 1-3 bytes if available
        if remaining_bytes == 1:
            j = (
                ((j ^ (MurMurHashUtil.get_unsigned_int(byte_arr[position]) & 0xFF)) & 0xFFFFFFFF) * 1540483477
            ) & 0xFFFFFFFF
        elif remaining_bytes == 2:
            j = (j ^ ((MurMurHashUtil.get_unsigned_int(byte_arr[position + 1]) & 0xFF) << 8)) & 0xFFFFFFFF
            j = (
                ((j ^ (MurMurHashUtil.get_unsigned_int(byte_arr[position]) & 0xFF)) & 0xFFFFFFFF) * 1540483477
            ) & 0xFFFFFFFF
        elif remaining_bytes == 3:
            j = (j ^ ((MurMurHashUtil.get_unsigned_int(byte_arr[position + 2]) & 0xFF) << 16)) & 0xFFFFFFFF
            j = (j ^ ((MurMurHashUtil.get_unsigned_int(byte_arr[position + 1]) & 0xFF) << 8)) & 0xFFFFFFFF
            j = (
                ((j ^ (MurMurHashUtil.get_unsigned_int(byte_arr[position]) & 0xFF)) & 0xFFFFFFFF) * 1540483477
            ) & 0xFFFFFFFF

        # Final mixing
        multiplier = 1540483477
        j5 = (((remaining ^ (j >> 18)) & 0xFFFFFFFF) * multiplier) & 0xFFFFFFFF
        j6 = (((j ^ (j5 >> 22)) & 0xFFFFFFFF) * multiplier) & 0xFFFFFFFF
        j7 = (((j5 ^ (j6 >> 17)) & 0xFFFFFFFF) * multiplier) & 0xFFFFFFFF
        j8 = j7 << 32

        return j8 | ((((j6 ^ (j7 >> 19)) & 0xFFFFFFFF) * multiplier) & 0xFFFFFFFF)

    @staticmethod
    def hash_unsigned(s: str | bytes) -> None:
        if isinstance(s, str):
            return MurMurHashUtil.read_unsigned_long(MurMurHashUtil.hash(s.encode()))
        elif isinstance(s, bytes) or isinstance(s, bytearray):
            return MurMurHashUtil.read_unsigned_long(MurMurHashUtil.hash(s))
        return None

    @staticmethod
    def long2bytes(value: int) -> bytes:
        # Convert long to 8 bytes in little-endian order
        result = bytearray(8)
        for i in range(8):
            result[7 - i] = (value >> (i * 8)) & 0xFF
        return bytes(result)

    @staticmethod
    def read_unsigned_long(value: int) -> int:
        return value & 0x7FFFFFFFFFFFFFFF

    @staticmethod
    def hash_unsigned_list(long_list: list[int]):
        byte_arr = b""
        for i in range(len(long_list)):
            if i == 0:
                byte_arr = MurMurHashUtil.long2bytes(long_list[i])
            else:
                byte_arr += MurMurHashUtil.long2bytes(long_list[i])

        return MurMurHashUtil.read_unsigned_long(MurMurHashUtil.hash(byte_arr))

    @staticmethod
    def hash_string(s: str):
        return MurMurHashUtil.hash(s.encode())



"""MowingDevice class to wrap around the betterproto dataclasses."""

from dataclasses import dataclass, field

import betterproto
from mashumaro.mixins.orjson import DataClassORJSONMixin

from pymammotion.data.model import HashList, RapidState
from pymammotion.data.model.device_info import DeviceFirmwares, DeviceNonWorkingHours, MowerInfo
from pymammotion.data.model.location import Location
from pymammotion.data.model.report_info import ReportData
from pymammotion.data.model.work import CurrentTaskSettings
from pymammotion.data.mqtt.properties import ThingPropertiesMessage
from pymammotion.data.mqtt.status import ThingStatusMessage
from pymammotion.http.model.http import CheckDeviceVersion, ErrorInfo
from pymammotion.proto import DeviceFwInfo, MowToAppInfoT, ReportInfoData, SystemRapidStateTunnelMsg, SystemUpdateBufMsg
from pymammotion.utility.constant import WorkMode
from pymammotion.utility.conversions import parse_double
from pymammotion.utility.map import CoordinateConverter


@dataclass
class MowingDevice(DataClassORJSONMixin):
    """Wraps the betterproto dataclasses, so we can bypass the groups for keeping all data."""

    name: str = ""
    online: bool = True
    enabled: bool = True
    update_check: CheckDeviceVersion = field(default_factory=CheckDeviceVersion)
    mower_state: MowerInfo = field(default_factory=MowerInfo)
    mqtt_properties: ThingPropertiesMessage | None = None
    status_properties: ThingStatusMessage | None = None
    map: HashList = field(default_factory=HashList)
    work: CurrentTaskSettings = field(default_factory=CurrentTaskSettings)
    location: Location = field(default_factory=Location)
    mowing_state: RapidState = field(default_factory=RapidState)
    report_data: ReportData = field(default_factory=ReportData)
    device_firmwares: DeviceFirmwares = field(default_factory=DeviceFirmwares)
    err_code_list: list = field(default_factory=list)
    err_code_list_time: list | None = field(default_factory=list)
    error_codes: dict[str, ErrorInfo] = field(default_factory=dict)
    non_work_hours: DeviceNonWorkingHours = field(default_factory=DeviceNonWorkingHours)

    def buffer(self, buffer_list: SystemUpdateBufMsg) -> None:
        """Update the device based on which buffer we are reading from."""
        match buffer_list.update_buf_data[0]:
            case 1:
                # 4 speed?
                if buffer_list.update_buf_data[5] != 0:
                    self.location.RTK.latitude = parse_double(buffer_list.update_buf_data[5], 8.0)
                    self.location.RTK.longitude = parse_double(buffer_list.update_buf_data[6], 8.0)
                if buffer_list.update_buf_data[7] != 0:
                    self.location.dock.latitude = parse_double(buffer_list.update_buf_data[7], 4.0)
                    self.location.dock.longitude = parse_double(buffer_list.update_buf_data[8], 4.0)
                    self.location.dock.rotation = buffer_list.update_buf_data[3] + 180
            case 2:
                self.err_code_list.clear()
                self.err_code_list_time.clear()
                self.err_code_list.extend(
                    [
                        buffer_list.update_buf_data[3],
                        buffer_list.update_buf_data[5],
                        buffer_list.update_buf_data[7],
                        buffer_list.update_buf_data[9],
                        buffer_list.update_buf_data[11],
                        buffer_list.update_buf_data[13],
                        buffer_list.update_buf_data[15],
                        buffer_list.update_buf_data[17],
                        buffer_list.update_buf_data[19],
                        buffer_list.update_buf_data[21],
                    ]
                )
                self.err_code_list_time.extend(
                    [
                        buffer_list.update_buf_data[4],
                        buffer_list.update_buf_data[6],
                        buffer_list.update_buf_data[8],
                        buffer_list.update_buf_data[10],
                        buffer_list.update_buf_data[12],
                        buffer_list.update_buf_data[14],
                        buffer_list.update_buf_data[16],
                        buffer_list.update_buf_data[18],
                        buffer_list.update_buf_data[20],
                        buffer_list.update_buf_data[22],
                    ]
                )

    def update_report_data(self, toapp_report_data: ReportInfoData) -> None:
        coordinate_converter = CoordinateConverter(self.location.RTK.latitude, self.location.RTK.longitude)
        for index, location in enumerate(toapp_report_data.locations):
            if index == 0 and location.real_pos_y != 0:
                self.location.position_type = location.pos_type
                self.location.orientation = int(location.real_toward / 10000)
                self.location.device = coordinate_converter.enu_to_lla(
                    parse_double(location.real_pos_y, 4.0), parse_double(location.real_pos_x, 4.0)
                )
                if location.zone_hash:
                    self.location.work_zone = (
                        location.zone_hash if self.report_data.dev.sys_status == WorkMode.MODE_WORKING else 0
                    )
                # if location.bol_hash:
                #     for loc in toapp_report_data.locations:
                #         if loc.bol_hash:
                #             if loc.bol_hash != location.bol_hash:
                #                 self.map = HashList()
                # MurMurHashUtil.hash_unsigned_list(list(self.map.area.keys()))

        if toapp_report_data.fw_info:
            self.update_device_firmwares(toapp_report_data.fw_info)

        self.report_data.update(toapp_report_data.to_dict(casing=betterproto.Casing.SNAKE))

    def run_state_update(self, rapid_state: SystemRapidStateTunnelMsg) -> None:
        coordinate_converter = CoordinateConverter(self.location.RTK.latitude, self.location.RTK.longitude)
        self.mowing_state = RapidState().from_raw(rapid_state.rapid_state_data)
        self.location.position_type = self.mowing_state.pos_type
        self.location.orientation = int(self.mowing_state.toward / 10000)
        self.location.device = coordinate_converter.enu_to_lla(
            parse_double(self.mowing_state.pos_y, 4.0), parse_double(self.mowing_state.pos_x, 4.0)
        )
        if self.mowing_state.zone_hash:
            self.location.work_zone = (
                self.mowing_state.zone_hash if self.report_data.dev.sys_status == WorkMode.MODE_WORKING else 0
            )

    def mow_info(self, toapp_mow_info: MowToAppInfoT) -> None:
        pass

    def report_missing_data(self) -> None:
        """Report missing data so we can refetch it."""

    def update_device_firmwares(self, fw_info: DeviceFwInfo) -> None:
        """Sets firmware versions on all parts of the robot or RTK."""
        for mod in fw_info.mod:
            match mod.type:
                case 1:
                    self.device_firmwares.main_controller = mod.version
                case 3:
                    self.device_firmwares.left_motor_driver = mod.version
                case 4:
                    self.device_firmwares.right_motor_driver = mod.version
                case 5:
                    self.device_firmwares.rtk_rover_station = mod.version
                case 101:
                    # RTK main board
                    self.device_firmwares.main_controller = mod.version
                case 102:
                    self.device_firmwares.rtk_version = mod.version
                case 103:
                    self.device_firmwares.lora_version = mod.version



"""data models"""

from .generate_route_information import GenerateRouteInformation
from .hash_list import HashList
from .rapid_state import RapidState, RTKStatus
from .region_data import RegionData

__all__ = ["GenerateRouteInformation", "HashList", "RapidState", "RTKStatus", "RegionData"]



from dataclasses import dataclass

from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class Camera(DataClassORJSONMixin):
    cameraId: int
    token: str


@dataclass
class StreamSubscriptionResponse(DataClassORJSONMixin):
    appid: str
    cameras: list[Camera]
    channelName: str
    token: str
    uid: int
    license: str | None = None
    availableTime: int | None = None


@dataclass
class VideoResourceResponse(DataClassORJSONMixin):
    id: str
    deviceId: str
    deviceName: str
    cycleType: int
    usageYearMonth: str
    totalTime: int
    availableTime: int



import csv
from typing import cast

from aiohttp import ClientSession

from pymammotion.const import MAMMOTION_API_DOMAIN, MAMMOTION_CLIENT_ID, MAMMOTION_CLIENT_SECRET, MAMMOTION_DOMAIN
from pymammotion.http.encryption import EncryptionUtils
from pymammotion.http.model.camera_stream import StreamSubscriptionResponse, VideoResourceResponse
from pymammotion.http.model.http import CheckDeviceVersion, ErrorInfo, LoginResponseData, Response
from pymammotion.http.model.response_factory import response_factory


class MammotionHTTP:
    def __init__(self) -> None:
        self.code = None
        self.msg = None
        self.account = None
        self._password = None
        self.response: Response | None = None
        self.login_info: LoginResponseData | None = None
        self._headers = {"User-Agent": "okhttp/4.9.3", "App-Version": "Home Assistant,1.14.2.29"}
        self.encryption_utils = EncryptionUtils()

    @staticmethod
    def generate_headers(token: str) -> dict:
        return {"Authorization": f"Bearer {token}"}

    async def login_by_email(self, email: str, password: str) -> Response[LoginResponseData]:
        return await self.login(email, password)

    async def get_all_error_codes(self) -> dict[str, ErrorInfo]:
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/user-server/v1/code/record/export-data",
                headers=self._headers,
            ) as resp:
                data = await resp.json()
                reader = csv.DictReader(data.get("data", "").split("\n"), delimiter=",")
                codes = dict()
                for row in reader:
                    error_info = ErrorInfo(**cast(dict, row))
                    codes[error_info.code] = error_info
                return codes

    async def oauth_check(self) -> Response:
        """Check if token is valid.

        Returns 401 if token is invalid. We then need to re-authenticate, can try to refresh token first
        """
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post("/user-server/v1/user/oauth/check", headers=self._headers) as resp:
                data = await resp.json()
                return Response.from_dict(data)

    async def pair_devices_mqtt(self, mower_name: str, rtk_name: str) -> Response:
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/device-server/v1/iot/device/pairing",
                headers=self._headers,
                json={"mowerName": mower_name, "rtkName": rtk_name},
            ) as resp:
                data = await resp.json()
                if data.get("status") == 200:
                    print(data)
                    return Response.from_dict(data)
                else:
                    print(data)

    async def unpair_devices_mqtt(self, mower_name: str, rtk_name: str) -> Response:
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/device-server/v1/iot/device/unpairing",
                headers=self._headers,
                json={"mowerName": mower_name, "rtkName": rtk_name},
            ) as resp:
                data = await resp.json()
                if data.get("status") == 200:
                    print(data)
                    return Response.from_dict(data)
                else:
                    print(data)

    async def net_rtk_enable(self, device_id: str) -> Response:
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/device-server/v1/iot/net-rtk/enable", headers=self._headers, json={"deviceId": device_id}
            ) as resp:
                data = await resp.json()
                if data.get("status") == 200:
                    print(data)
                    return Response.from_dict(data)
                else:
                    print(data)

    async def get_stream_subscription(self, iot_id: str) -> Response[StreamSubscriptionResponse]:
        """Fetches stream subscription data from agora.io for a given IoT device."""
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/device-server/v1/stream/subscription",
                json={"deviceId": iot_id},
                headers={
                    **self._headers,
                    "Authorization": f"Bearer {self.login_info.access_token}",
                    "Content-Type": "application/json",
                    "User-Agent": "okhttp/4.9.3",
                },
            ) as resp:
                data = await resp.json()
                # TODO catch errors from mismatch like token expire etc
                # Assuming the data format matches the expected structure
                response = Response[StreamSubscriptionResponse].from_dict(data)
                if response.code != 0:
                    return response
                response.data = StreamSubscriptionResponse.from_dict(data.get("data", {}))
                return response

    async def get_stream_subscription_mini_or_x_series(
        self, iot_id: str, is_yuka: bool
    ) -> Response[StreamSubscriptionResponse]:
        # Prepare the payload with cameraStates based on is_yuka flag
        """Fetches stream subscription data for a given IoT device."""
        payload = {"deviceId": iot_id, "mode": 0, "cameraStates": []}

        # Add appropriate cameraStates based on the is_yuka flag
        if is_yuka:
            payload["cameraStates"] = [{"cameraState": 1}, {"cameraState": 0}, {"cameraState": 1}]
        else:
            payload["cameraStates"] = [{"cameraState": 1}, {"cameraState": 0}, {"cameraState": 0}]

        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/device-server/v1/stream/token",
                json=payload,
                headers={
                    **self._headers,
                    "Authorization": f"Bearer {self.login_info.access_token}",
                    "Content-Type": "application/json",
                    "User-Agent": "okhttp/4.9.3",
                },
            ) as resp:
                data = await resp.json()
                # TODO catch errors from mismatch like token expire etc
                # Assuming the data format matches the expected structure
                response = Response[StreamSubscriptionResponse].from_dict(data)
                if response.code != 0:
                    return response
                response.data = StreamSubscriptionResponse.from_dict(data.get("data", {}))
                return response

    async def get_video_resource(self, iot_id: str) -> Response[VideoResourceResponse]:
        """Fetch video resource for a given IoT ID."""
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.get(
                f"/device-server/v1/video-resource/{iot_id}",
                headers={
                    "Authorization": f"Bearer {self.login_info.access_token}",
                    "Content-Type": "application/json",
                    "User-Agent": "okhttp/4.9.3",
                },
            ) as resp:
                data = await resp.json()
                # TODO catch errors from mismatch like token expire etc
                # Assuming the data format matches the expected structure
                response = Response[VideoResourceResponse].from_dict(data)
                if response.code != 0:
                    return response
                response.data = VideoResourceResponse.from_dict(data.get("data", {}))
                return response

    async def get_device_ota_firmware(self, iot_ids: list[str]) -> Response[list[CheckDeviceVersion]]:
        """Checks device firmware versions for a list of IoT IDs."""
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/device-server/v1/devices/version/check",
                json={"deviceIds": iot_ids},
                headers={
                    **self._headers,
                    "Authorization": f"Bearer {self.login_info.access_token}",
                    "Content-Type": "application/json",
                    "User-Agent": "okhttp/4.9.3",
                },
            ) as resp:
                data = await resp.json()
                # TODO catch errors from mismatch like token expire etc
                # Assuming the data format matches the expected structure
                return response_factory(Response[list[CheckDeviceVersion]], data)

    async def start_ota_upgrade(self, iot_id: str, version: str) -> Response[str]:
        """Initiates an OTA upgrade for a device."""
        async with ClientSession(MAMMOTION_API_DOMAIN) as session:
            async with session.post(
                "/device-server/v1/ota/device/upgrade",
                json={"deviceId": iot_id, "version": version},
                headers={
                    **self._headers,
                    "Authorization": f"Bearer {self.login_info.access_token}",
                    "Content-Type": "application/json",
                    "User-Agent": "okhttp/4.9.3",
                },
            ) as resp:
                data = await resp.json()
                # TODO catch errors from mismatch like token expire etc
                # Assuming the data format matches the expected structure
                return response_factory(Response[str], data)

    async def refresh_login(self, account: str, password: str | None = None) -> Response[LoginResponseData]:
        if self._password is None and password is not None:
            self._password = password
        if self._password is None:
            raise ValueError("Password is required for refresh login")
        return await self.login(account, self._password)

    async def login(self, account: str, password: str) -> Response[LoginResponseData]:
        """Logs in to the service using provided account and password."""
        self.account = account
        self._password = password
        async with ClientSession(MAMMOTION_DOMAIN) as session:
            async with session.post(
                "/oauth/token",
                headers={
                    **self._headers,
                    "Encrypt-Key": self.encryption_utils.encrypt_by_public_key(),
                    "Decrypt-Type": "3",
                    "Ec-Version": "v1",
                },
                params=dict(
                    username=self.encryption_utils.encryption_by_aes(account),
                    password=self.encryption_utils.encryption_by_aes(password),
                    client_id=self.encryption_utils.encryption_by_aes(MAMMOTION_CLIENT_ID),
                    client_secret=self.encryption_utils.encryption_by_aes(MAMMOTION_CLIENT_SECRET),
                    grant_type=self.encryption_utils.encryption_by_aes("password"),
                ),
            ) as resp:
                if resp.status != 200:
                    print(resp.json())
                    return Response.from_dict({"code": resp.status, "msg": "Login failed"})
                data = await resp.json()
                login_response = response_factory(Response[LoginResponseData], data)
                if login_response.data is None:
                    print(login_response)
                    return Response.from_dict({"code": resp.status, "msg": "Login failed"})
                self.login_info = login_response.data
                self._headers["Authorization"] = (
                    f"Bearer {self.login_info.access_token}" if login_response.data else None
                )
                self.response = login_response
                self.msg = login_response.msg
                self.code = login_response.code
                # TODO catch errors from mismatch user / password elsewhere
                # Assuming the data format matches the expected structure
                return login_response



from dataclasses import dataclass

from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class Device(DataClassORJSONMixin):
    gmtModified: int
    netType: str
    categoryKey: str
    productKey: str
    nodeType: str
    isEdgeGateway: bool
    deviceName: str
    categoryName: str
    identityAlias: str
    productName: str
    iotId: str
    bindTime: int
    owned: int
    identityId: str
    thingType: str
    status: int
    nickName: str | None = None
    description: str | None = None
    productImage: str | None = None
    categoryImage: str | None = None
    productModel: str | None = None

    class Config(BaseConfig):
        omit_default = True


@dataclass
class Data(DataClassORJSONMixin):
    total: int
    data: list[Device]
    pageNo: int
    pageSize: int


@dataclass
class ListingDevByAccountResponse(DataClassORJSONMixin):
    code: int
    data: Data | None
    id: str | None = None
